import * as i0 from '@angular/core';
import { Injectable, Component, ChangeDetectionStrategy, Input, NgModule } from '@angular/core';
import { createCustomElement } from '@angular/elements';
import * as i1 from 'formbird-services';
import { ClientConstants, FormbirdServicesModule } from 'formbird-services';
import * as Handlebars from 'handlebars';
import * as uuid from 'uuid';
import * as i1$1 from '@angular/common/http';
import { HttpHeaders } from '@angular/common/http';
import * as moment from 'moment';
import { Subscription } from 'rxjs';
import * as i2 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i3 from '@angular/material/icon';
import { MatIconModule } from '@angular/material/icon';
import { FormControl, Validators, FormGroup } from '@angular/forms';
import { MatAutocompleteModule } from '@angular/material/autocomplete';
import { __awaiter } from 'tslib';
import { chunk, nth, isEmpty, toLower, isArray, cloneDeep, get, includes, isEqual, isPlainObject, map, orderBy } from 'lodash';
import { SharedConstants } from '@formbird/types';
import { debounceTime } from 'rxjs/operators';

class CustomElementService {
    constructor(fbInjectorService) {
        this.fbInjectorService = fbInjectorService;
        this.convertToCustomElement = (component, selector, injector, angularSuffix, version, options) => {
            // if angularJS is detected, append the suffix to selector to avoid duplicated component selector
            if (angularSuffix && this.fbInjectorService.isAngularJSCore()) {
                selector += angularSuffix;
            }
            if (!version) {
                version = 'NOT-SET';
            }
            if (!customElements.get(selector)) {
                const scComponent = createCustomElement(component, { injector: injector });
                const connectedCallback = scComponent.prototype.connectedCallback || function () { };
                scComponent.prototype.connectedCallback = function () {
                    this.setAttribute('fb-component-version', version);
                    if (options) {
                        this.setAttribute('is-static-panel', options.isStaticPanel);
                        this.setAttribute('is-collapsible-panel', options.isCollapsiblePanel);
                    }
                    connectedCallback.call(this);
                };
                customElements.define(selector, scComponent);
            }
        };
    }
}
/** @nocollapse */ /** @nocollapse */ CustomElementService.ɵfac = function CustomElementService_Factory(t) { return new (t || CustomElementService)(i0.ɵɵinject(i1.FormbirdInjectorService)); };
/** @nocollapse */ /** @nocollapse */ CustomElementService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: CustomElementService, factory: CustomElementService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(CustomElementService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return [{ type: i1.FormbirdInjectorService }]; }, null);
})();

class UtilSharingService {
    constructor(clientRulesService) {
        this.clientRulesService = clientRulesService;
    }
    // Function that would trigger/call functions from one component to other components/ rules
    //  componentFieldName=>name of component that contains the function;functionName=> name of the function declared in componentFieldName
    // e.g component function call:scope.sclUtilSharing.fnCallFunction(scope.fieldName,'eventReloadDatatable');
    // e.g rules calling: UtilSharingLibrary.fnCallFunction('scDatatablesTest','eventReloadDatatable');
    fnCallFunction(componentFieldName, functionName, param1, param2) {
        this.clientRulesService.setSharedValue(functionName, { fieldName: componentFieldName, param1, param2, time: new Date() });
        if (/get/.test(functionName.slice(0, 3))) {
            const objectData = this.clientRulesService.getSharedValue(functionName.replace(/^get/, 'set'));
            let objectDataReturn = [];
            if (objectData) {
                if (objectData.param1 && objectData.param2) {
                    objectDataReturn.push(objectData.param1, objectData.param2);
                }
                else if (objectData.param1) {
                    objectDataReturn = objectData.param1;
                }
                else {
                    objectDataReturn = objectData;
                }
                return objectDataReturn;
            }
        }
    }
    // Function that would register a list of function to call in fnCallFunction
    // listOfFunctions=>['eventReloadDatatable','eventReloadTable','function1','...']
    // list of functions that will be available  in fnCallFunction to be called.
    fnRegisterFunctions(listOfFunctions) {
        for (let i = 0, n = listOfFunctions.length; i < n; i++) {
            this.fnCreateWatchForFunctionCall(listOfFunctions[i]);
        }
    }
    fnCreateWatchForFunctionCall(functionName) {
        console.log('--------TODO implment watch in UtilSharingService');
        //  scope.$watch(function() {return this.clientRulesService.getSharedValue(functionName); }, function(objValue) {
        //   if (objValue) {
        //      // logger.info(functionName," is watch",objValue,objValue.fieldName,scope.fieldName);
        //      if (scope.fieldName === objValue.fieldName) {
        //       let variableFunction = 'scope.'+ functionName;
        //       let fn = eval('(' + variableFunction + ')');
        //       // logger.info(objValue.fieldName,scope.fieldName,variableFunction,fn);
        //       if (objValue.param1 && objValue.param2) {
        //         // variableFunction = "scope."+functionName+"('"+objValue.param1+"','"+objValue.param2+"')";
        //         fn(objValue.param1, objValue.param2);
        //       } else if (objValue.param1) {
        //          // variableFunction = "scope."+functionName+"('"+objValue.param1+"')";
        //          fn(objValue.param1);
        //       } else {
        //           fn();
        //       }
        //       // eval("(" + variableFunction + ")");
        //      }
        //    }
        //  }, true);
        //  let tempName = functionName;
        //  let firstThree = tempName.slice(0, 3);
        //  if (/set/.test(firstThree)) {
        //   let newStr = tempName.replace(firstThree,'');
        //   newStr = newStr.substr(0, 1).toLowerCase() + newStr.substr(1, newStr.length - 1);
        //   scope.$watch(newStr, function(objValue) {
        //      if (objValue) {
        //       logger.info(newStr,' is modified', objValue);
        //       ClientRulesService.setSharedValue(functionName, {fieldName: scope.fieldName, param1: objValue, time: new Date()});
        //      }
        //    }, true);
        //  }
    }
    fnPassedDataObject(fieldName, objectName, objectDataValue) {
        this.clientRulesService.setSharedValue(objectName, { fieldName, objectDataValue, time: new Date().getTime() });
    }
    fnGetDataObject(fieldName, objectName) {
        const objectData = this.clientRulesService.getSharedValue(objectName);
        return objectData.fieldName === fieldName ? objectData : null;
    }
}
/** @nocollapse */ /** @nocollapse */ UtilSharingService.ɵfac = function UtilSharingService_Factory(t) { return new (t || UtilSharingService)(i0.ɵɵinject(i1.ClientRulesService)); };
/** @nocollapse */ /** @nocollapse */ UtilSharingService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: UtilSharingService, factory: UtilSharingService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(UtilSharingService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return [{ type: i1.ClientRulesService }]; }, null);
})();

class SclHandlebarsService {
    constructor() {
    }
    /////////
    initHandlebars(helpers) {
        // Adding some standard helpers
        // Comparison
        Handlebars.registerHelper('ifEquals', function (arg1, arg2, options) {
            return (arg1 == arg2) ? options.fn(this) : options.inverse(this);
        });
        Handlebars.registerHelper('ifNotEquals', function (arg1, arg2, options) {
            return (arg1 != arg2) ? options.fn(this) : options.inverse(this);
        });
        Handlebars.registerHelper('switch', function (value, options) {
            this.switch_value = value;
            this.switch_break = false;
            return options.fn(this);
        });
        Handlebars.registerHelper('case', function (value, options) {
            if (value == this.switch_value) {
                this.switch_break = true;
                return options.fn(this);
            }
        });
        Handlebars.registerHelper('default', function (value, options) {
            if (this.switch_break == false) {
                return value;
            }
        });
        // Date handling
        // Handlebars.registerHelper('dateFormat', (date, format, options) => moment(date).format(format));
        // Handlebars.registerHelper('dateAdd', (date, amount, what, options) => moment(date).add(amount, what).format());
        // Handlebars.registerHelper('dateFromNow', (date, options) => moment(date).fromNow());
        // Handlebars.registerHelper('dateToNow', (date, options) => moment(date).toNow());
        // Handlebars.registerHelper('dateCalendar', (date, options) => moment(date).calendar());
        // If component has helpers defined add them in too
        if (typeof helpers !== 'undefined' && helpers !== null) {
            for (let i = 0; i < helpers.length; i++) {
                if (typeof helpers[i].name === 'string' && typeof helpers[i].function === 'string') {
                    let helpFunc;
                    eval('helpFunc = ' + helpers[i].function);
                    Handlebars.registerHelper(helpers[i].name, helpFunc);
                }
            }
        }
        // console.log("%c initHandlebars, registerHelper :", 'color:blue');
        // console.log(Handlebars.helpers);
    }
    checkRegistedHelpers(helpers) {
        if (!helpers) {
            return;
        }
        // console.log('%c ibefore checkRegistedHelpers : ', 'color:blue');
        // console.log(Handlebars.helpers);
        let isRegistered = true;
        for (let i = 0; i < helpers.length; i++) {
            if (typeof helpers[i].name === 'string' && typeof helpers[i].function === 'string') {
                if (!(helpers[i].name in Handlebars.helpers)) {
                    isRegistered = false;
                    break;
                }
            }
        }
        if (!isRegistered) {
            this.initHandlebars(helpers);
        }
    }
    processHandlebars(handlebarsString, data, helpers) {
        //   this.checkRegistedHelpers(helpers);
        // if(handlebarsString == null) return "";
        // var hStr;
        // if(typeof handlebarsString === "string")
        //     hStr = handlebarsString;
        // else
        //     hStr = JSON.stringify(handlebarsString);
        // // begin : 14274
        // var handlebarsContext = {
        //     document: this.document, // the fields from the current document
        //     doc: scope.doc, // a duplicate for compatibility of document
        //     account: scope.account, // the account document from scope
        //     tpl: scope.tpl, // the template for the current document
        //     tplItem: scope.tplItem // the current components configuration details
        //   };
        //   var invalidField = ["document", "doc", "account", "tpl", "tplItem", "results", "resultsCount", "resultsIndex", "result", "row"];
        //   if(data){
        //       if(Object(data) === data){ // if the second parameter is object has property
        //           for(var f in data){
        //               if(f === "queryResultDocument"){
        //                   handlebarsContext["row"] = data[f];
        //                   handlebarsContext["result"] = data[f];
        //                   for(var j in data[f]){
        //                       if(invalidField.indexOf(j) > -1){
        //                           logger.info("Handlebars Context Error: the field " + j + " is already present in document documentId so could not be set. It's values will NOT be available to handlebars expressions. Please advise your Application Developer to not use this field name if possible");
        //                           continue;
        //                       }
        //                       handlebarsContext[j] = data[f][j];
        //                   }
        //               }
        //               else{
        //                   if(handlebarsContext.hasOwnProperty(f)){
        //                       logger.info("Handlebars Context Error: the field " + f + " is already present in document documentId so could not be set. It's values will NOT be available to handlebars expressions. Please advise your Application Developer to not use this field name if possible");
        //                           continue;
        //                   }
        //                   handlebarsContext[f] = data[f];
        //               }
        //           }
        //       }
        //       else{
        //           logger.info("Handlebars Context Error: object " + data + " wasn't passed in and the parameters is being ignored");
        //       }
        //   }
        // // end : 14274
        // //console.log('handlebarsContext = ' + JSON.stringify(handlebarsContext));
        // var result = null;
        // try{
        //     // console.log(Handlebars.helpers);
        //     var template = Handlebars.compile(hStr);
        //       result = template(handlebarsContext);
        // }
        // catch(err){
        //       err.hStr = hStr;
        //     logger.info(err);
        // }
        //   return result;
        return;
    }
    processIntHandlebars(handlebarsString, data, helpers) {
        // const hStr = this.processHandlebars(handlebarsString, data, helpers);
        // if (hStr) {
        //     return parseInt( hStr );
        // } else {
        //   return 0;
        // }
    }
    processFloatHandlebars(handlebarsString, data, helpers) {
        //   var hStr = this.processHandlebars(handlebarsString, data, helpers);
        //   if(hStr){
        //       return parseFloat( hStr );
        //   }
        // else return 0;
    }
    processBooleanHandlebars(handlebarsString, data, helpers) {
        // return  this.processHandlebars(handlebarsString, data, helpers) === 'true';
    }
    processArrayHandlebars(handlebarsString, data, helpers) {
        // var hStr = this.processHandlebars(handlebarsString, data, helpers);
        // if(hStr){
        //     try{
        //         var obj = JSON.parse( hStr );
        //         var arr = [];
        //         for(var i in obj){
        //             arr.push(obj[i]);
        //         }
        //         return arr;
        //     }
        //     catch(err){
        //         return null;
        //     }
        // }
        // else{
        //     return null;
        // }
    }
}
/** @nocollapse */ /** @nocollapse */ SclHandlebarsService.ɵfac = function SclHandlebarsService_Factory(t) { return new (t || SclHandlebarsService)(); };
/** @nocollapse */ /** @nocollapse */ SclHandlebarsService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclHandlebarsService, factory: SclHandlebarsService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclHandlebarsService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return []; }, null);
})();

const defaultHelpers = [
    {
        name: 'ifEquals',
        function: `function(arg1, arg2, options) {return (arg1 === arg2) ? options.fn(this) : options.inverse(this);}`
    },
    {
        name: 'ifNotEquals',
        function: `function(arg1, arg2, options) {return (arg1 !== arg2) ? options.fn(this) : options.inverse(this);}`
    },
    {
        name: 'switch',
        function: `function(value, options) {this.switch_value = value;this.switch_break = false; return options.fn(this);}`
    },
    {
        name: 'case',
        function: `function(value, options) {if (value === this.switch_value) {this.switch_break = true;return options.fn(this);}}`
    },
    {
        name: 'default',
        function: `function(value, options) {if (this.switch_break === false) {return value;}}`
    },
    {
        name: 'dateFormat',
        function: `function(date, format, options) { return moment(date).format(format); }`
    },
    {
        name: 'dateAdd',
        function: `function(date, amount, what, options) { return moment(date).add(amount, what).format(); }`
    },
    {
        name: 'dateFromNow',
        function: `function(date, options) { return moment(date).fromNow(); }`
    },
    {
        name: 'dateToNow',
        function: `function(date, options) { return moment(date).toNow(); }`
    },
    {
        name: 'dateCalendar',
        function: `function(date, options) { return moment(date).calendar(); }`
    }
];
const sclHandlebars = (() => {
    return {
        initHandlebars(helpers) {
            /*
            // Adding some standard helpers
            // Comparison
            Handlebars.registerHelper('ifEquals', function(arg1, arg2, options) {
                return (arg1 === arg2) ? options.fn(this) : options.inverse(this);
            });
            Handlebars.registerHelper('ifNotEquals', function(arg1, arg2, options) {
                return (arg1 !== arg2) ? options.fn(this) : options.inverse(this);
            });
            Handlebars.registerHelper('switch', function(value, options) {
                let that: any = {};
                that.switch_value = value;
                that.switch_break = false;
                return options.fn(that);
            });
            Handlebars.registerHelper('case', function(value, options) {
            if (value === this.switch_value) {
                this.switch_break = true;
                return options.fn(this);
            }
            });
            Handlebars.registerHelper('default', function(value, options) {
            if (this.switch_break === false) {
                return value;
            }
            });
            // Date handling
            Handlebars.registerHelper('dateFormat', function(date, format, options) { return moment(date).format(format); });
            Handlebars.registerHelper('dateAdd', function(date, amount, what, options) { return moment(date).add(amount, what).format(); });
            Handlebars.registerHelper('dateFromNow', function(date, options) { return moment(date).fromNow(); });
            Handlebars.registerHelper('dateToNow', function(date, options) { return moment(date).toNow(); });
            Handlebars.registerHelper('dateCalendar', function(date, options) { return moment(date).calendar(); }); */
            for (let i = 0; i < defaultHelpers.length; i++) {
                if (typeof defaultHelpers[i].name === 'string' && typeof defaultHelpers[i].function === 'string') {
                    let defaultHelps = new Function('return ' + defaultHelpers[i].function)();
                    Handlebars.registerHelper(defaultHelpers[i].name, defaultHelps);
                }
            }
            // If component has helpers defined add them in too
            if (typeof helpers !== 'undefined' && helpers !== null) {
                for (const helper of helpers) {
                    if (typeof helper.name === 'string' && typeof helper.function === 'string') {
                        let helpFunc = helper.function;
                        if (typeof helpFunc === 'string') {
                            helpFunc = new Function('return ' + helpFunc)();
                        }
                        Handlebars.registerHelper(helper.name, helpFunc);
                    }
                }
            }
        },
        defaultHelpers() {
            return defaultHelpers;
        },
        checkRegistedHelpers(helpers) {
            if (!helpers) {
                return;
            }
            // console.log('%c ibefore checkRegistedHelpers : ', 'color:blue');
            // console.log(Handlebars.helpers);
            let isRegistered = true;
            for (let i = 0; i < helpers.length; i++) {
                if (typeof helpers[i].name === 'string' && typeof helpers[i].function === 'string') {
                    if (!(helpers[i].name in Handlebars.helpers)) {
                        isRegistered = false;
                        break;
                    }
                }
            }
            if (!isRegistered) {
                sclHandlebars.initHandlebars(helpers);
            }
        },
        // throws an error instead of returning null
        processHandlebarsV2(handlebarsString, data, handlebarsContext, helpers) {
            sclHandlebars.checkRegistedHelpers(helpers);
            if (handlebarsString == null) {
                return '';
            }
            let hStr;
            if (typeof handlebarsString === 'string') {
                hStr = handlebarsString;
            }
            else {
                hStr = JSON.stringify(handlebarsString);
            }
            // begin : 14274
            if (!handlebarsContext) {
                handlebarsContext = {};
            }
            const invalidField = ['document', 'doc', 'account', 'tpl', 'componentDefinition',
                'results', 'resultsCount', 'resultsIndex', 'result', 'row'];
            if (data) {
                if (Object(data) === data) { // if the second parameter is object has property
                    for (const f in data) {
                        if (f === 'queryResultDocument') {
                            handlebarsContext.row = data[f];
                            handlebarsContext.result = data[f];
                            for (const j in data[f]) {
                                if (data[f].hasOwnProperty(j)) {
                                    if (invalidField.indexOf(j) > -1) {
                                        console.log('Handlebars Context Error: the field ' + j
                                            + ' is already present in document documentId so could not be set.'
                                            + ' It\'s values will NOT be available to handlebars expressions.'
                                            + ' Please advise your Application Developer to not use this field name if possible');
                                        continue;
                                    }
                                    handlebarsContext[j] = data[f][j];
                                }
                            }
                        }
                        else {
                            if (handlebarsContext.hasOwnProperty(f)) {
                                console.log('error : handlebarsString = ' + handlebarsString
                                    + '\n\t data = ' + JSON.stringify(data)
                                    + '\n\t handlebarsContext = ' + JSON.stringify(handlebarsContext)
                                    + '\n\t helpers = ' + JSON.stringify(helpers));
                                // console.log('Handlebars Context Error: the field ' + f
                                //  + ' is already present in document documentId so could not be set.'
                                //  + ' It\'s values will NOT be available to handlebars expressions.'
                                //  + ' Please advise your Application Developer to not use this field name if possible');
                                continue;
                            }
                            handlebarsContext[f] = data[f];
                        }
                    }
                }
                else {
                    console.log('Handlebars Context Error: object ' + data + ' wasn\'t passed in and the parameters is being ignored');
                }
            }
            // end : 14274
            // console.log('handlebarsContext = ' + JSON.stringify(handlebarsContext));
            let result = null;
            try {
                const template = Handlebars.compile(hStr);
                result = template(handlebarsContext);
            }
            catch (err) {
                err.hStr = hStr;
                throw err;
            }
            return result;
        },
        // backward compatibility purposes because it's still used in a lot of areas: returns an null object
        processHandlebars(handlebarsString, data, handlebarsContext, helpers) {
            let result = null;
            try {
                result = sclHandlebars.processHandlebarsV2(handlebarsString, data, handlebarsContext, helpers);
            }
            catch (err) {
                console.error(err);
            }
            return result;
        },
        processIntHandlebars(handlebarsString, data, helpers) {
            const hStr = sclHandlebars.processHandlebars(handlebarsString, data, helpers);
            if (hStr) {
                return parseInt(hStr, 0);
            }
            else {
                return 0;
            }
        },
        processFloatHandlebars(handlebarsString, data, helpers) {
            const hStr = sclHandlebars.processHandlebars(handlebarsString, data, helpers);
            if (hStr) {
                return parseFloat(hStr);
            }
            else {
                return 0;
            }
        },
        processBooleanHandlebars(handlebarsString, data, helpers) {
            return sclHandlebars.processHandlebars(handlebarsString, data, helpers) === 'true';
        },
        processArrayHandlebars(handlebarsString, data, helpers) {
            const hStr = sclHandlebars.processHandlebars(handlebarsString, data, helpers);
            if (hStr) {
                try {
                    const obj = JSON.parse(hStr);
                    const arr = [];
                    for (const i in obj) {
                        if (obj.hasOwnProperty(i)) {
                            arr.push(obj[i]);
                        }
                    }
                    return arr;
                }
                catch (err) {
                    return null;
                }
            }
            else {
                return null;
            }
        }
    };
})();

class SclAceEditorService {
    constructor() {
    }
    /////////
    enableFullscreen(aceEditor, fieldName) {
        const aced = document.getElementById(fieldName);
        const matIcon = document.createElement('mat-icon');
        const btn = document.createElement('button');
        matIcon.className = 'editor-resize-fullscreen-icon mat-icon notranslate material-icons mat-icon-no-color';
        matIcon.innerText = 'fullscreen';
        matIcon.title = 'Enter fullscreen';
        matIcon.id = 'editorFullScreenIcon';
        // btn.append(matIcon);
        btn.appendChild(matIcon);
        if (this.hasClass(aced, 'fullScreen')) {
            matIcon.innerText = 'fullscreen_exit';
            matIcon.title = 'Exit fullscreen';
        }
        this.editorStyle(aced);
        btn.className = 'gbl-fse-btn';
        btn.style.position = 'absolute';
        btn.style.top = '0px';
        btn.style.right = '0px';
        btn.style.opacity = '0.3';
        btn.style.zIndex = '1';
        btn.addEventListener('click', function (e) {
            const scrollBar = aced.getElementsByClassName('ace_scrollbar-v')[0];
            const editorFullScreenIcon = document.getElementById('editorFullScreenIcon');
            if (this.hasClass(aced, 'fullScreen')) {
                this.removeClass(aced, 'fullScreen');
                editorFullScreenIcon.innerText = 'fullscreen';
                editorFullScreenIcon.title = 'Enter fullscreen';
                if (scrollBar) {
                    scrollBar.style.display = 'none';
                }
                aceEditor.setAutoScrollEditorIntoView(true);
                aceEditor.setOptions({
                    maxLines: Infinity
                });
                btn.style.top = '0px';
                btn.style.right = '0px';
            }
            else {
                this.addClass(aced, 'fullScreen');
                editorFullScreenIcon.innerText = 'fullscreen_exit';
                editorFullScreenIcon.title = 'Exit fullscreen';
                if (scrollBar) {
                    scrollBar.style.display = null;
                }
                const lineGroup = aced.getElementsByClassName('ace_line_group')[0];
                let lineHeight;
                if (lineGroup) {
                    lineHeight = lineGroup.style.height;
                }
                const editorHeight = aced.offsetHeight; // parseInt(aced.offsetHeight);
                const maxLine = isNaN(Math.ceil(editorHeight / lineHeight)) ? 0 : Math.ceil(editorHeight / lineHeight);
                aced.style.height = editorHeight + 'px';
                aceEditor.setOptions({
                    maxLines: 0
                });
                aceEditor.setAutoScrollEditorIntoView(false);
                btn.style.top = '1px';
                btn.style.right = '20px';
            }
            aceEditor.resize();
            aceEditor.focus();
        }.bind(this));
        aced.appendChild(btn);
    }
    editorStyle(aced) {
        const styleNode = document.createElement('style');
        styleNode.type = 'text/css';
        let css = '.ace_editor.fullScreen { ' +
            'overflow-y:auto;height: auto!important;width: auto;' +
            'border: 0;margin: 0;position: fixed !important;' +
            'top: 0;bottom: 0;left: 0;right: 0;z-index: 9999;' +
            '}';
        css += '.ace_editor .gbl-fse-btn:hover{ opacity: 0.9!important; }';
        css += '.fullScreen{ overflow: hidden }';
        // browser detection (based on prototype.js)
        if (!!(window.attachEvent && !window.opera)) {
            // styleNode.styleSheet.cssText = css;//TO DO
        }
        else {
            const styleText = document.createTextNode(css);
            styleNode.appendChild(styleText);
        }
        aced.parentNode.appendChild(styleNode);
    }
    hasClass(el, clsname) {
        const classNames = el.className.split(' ');
        for (let i = 0; i < classNames.length; i++) {
            if (clsname === classNames[i]) {
                return true;
            }
        }
        return false;
    }
    addClass(el, clsname) {
        if (!this.hasClass(el, clsname)) {
            el.className = (el.className + ' ' + clsname);
        }
    }
    removeClass(el, clsname) {
        const classNames = el.className.split(' ');
        const newClasses = [];
        for (let i = 0; i < classNames.length; i++) {
            if (clsname !== classNames[i]) {
                newClasses.push(classNames[i]);
            }
        }
        el.className = newClasses.join(' ');
    }
}
/** @nocollapse */ /** @nocollapse */ SclAceEditorService.ɵfac = function SclAceEditorService_Factory(t) { return new (t || SclAceEditorService)(); };
/** @nocollapse */ /** @nocollapse */ SclAceEditorService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclAceEditorService, factory: SclAceEditorService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclAceEditorService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return []; }, null);
})();

class SclAceTextRulesInterfaceService {
    constructor() {
    }
    /////////
    syncEditBuffer(doc, fieldName, componentDefinition, _editor) {
        let content = doc[fieldName];
        if (componentDefinition.textFormat === 'raw-json') {
            try {
                content = JSON.parse(_editor.value);
            }
            catch (e) { }
        }
        else {
            content = _editor.value;
        }
        doc[fieldName] = content;
    }
    extractValuesToDoc(doc, fieldName, componentDefinition, _editor, tempKeys) {
        let text = _editor.value;
        if (componentDefinition.textFormat === 'json') {
            const editor = _editor.getEditor();
            const session = editor.session;
            const textValue = session.getValue();
            if (componentDefinition.storeToRoot) {
                if (textValue && textValue.length >= 6) {
                    try {
                        const json = JSON.parse(textValue);
                        for (let i = 0; i < tempKeys.length; i++) {
                            if (doc[tempKeys[i]]) {
                                delete doc[tempKeys[i]];
                            }
                        }
                        // angular.forEach(json, function(bean, key) {
                        //   this.tempKeys.push(key);
                        //   this.doc[key] = bean;
                        // });
                        Object.keys(json).forEach(function (k) {
                            // console.log(k + ' - ' + obj[k]);
                            this.tempKeys.push(k);
                            this.doc[k] = json[k];
                        });
                    }
                    catch (err) { }
                }
            }
            if (textValue) {
                text = JSON.stringify(textValue, undefined, 2);
            }
        }
        else if (componentDefinition.textFormat === 'raw-json') {
            doc[fieldName] = {};
            try {
                text = JSON.parse(text);
            }
            catch (err) { }
        }
        if (text !== '') {
            doc[fieldName] = text;
        }
    }
}
/** @nocollapse */ /** @nocollapse */ SclAceTextRulesInterfaceService.ɵfac = function SclAceTextRulesInterfaceService_Factory(t) { return new (t || SclAceTextRulesInterfaceService)(); };
/** @nocollapse */ /** @nocollapse */ SclAceTextRulesInterfaceService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclAceTextRulesInterfaceService, factory: SclAceTextRulesInterfaceService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclAceTextRulesInterfaceService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return []; }, null);
})();

class SclUtilUrlService {
    constructor(formbirdInjectorService, routerService) {
        this.formbirdInjectorService = formbirdInjectorService;
        this.routerService = routerService;
        this.documentModalService = this.formbirdInjectorService.get('DocumentModalService');
    }
    openLink(link, mode, componentScope) {
        const openMode = mode ? mode : 'currentWindow';
        switch (openMode) {
            case 'currentWindow':
                this.routerService.navigateByUrl(link);
                break;
            case 'newWindow':
                window.open(link, '_blank');
                break;
            case 'popup':
                window.open(link, undefined, 'height=300,width=600');
                break;
            case 'overlay':
                const documentId = this.getDocIdFromLink(link);
                componentScope.selectedDocumentId = documentId;
                this.documentModalService.openDocumentInModal(documentId, componentScope);
                break;
            case 'noDisplay':
                break;
            default:
                this.routerService.navigateByUrl(link);
                break;
        }
    }
    getDocIdFromLink(link) {
        const lstIdx = link.lastIndexOf('/');
        if (!link || lstIdx === -1) {
            return null;
        }
        else {
            return link.substr(lstIdx + 1, link.length);
        }
    }
}
/** @nocollapse */ /** @nocollapse */ SclUtilUrlService.ɵfac = function SclUtilUrlService_Factory(t) { return new (t || SclUtilUrlService)(i0.ɵɵinject(i1.FormbirdInjectorService), i0.ɵɵinject(i1.RouterService)); };
/** @nocollapse */ /** @nocollapse */ SclUtilUrlService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclUtilUrlService, factory: SclUtilUrlService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclUtilUrlService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return [{ type: i1.FormbirdInjectorService }, { type: i1.RouterService }]; }, null);
})();

class SclUtilFilterService {
    constructor() { }
    arrayFilter(obj, arg1) {
        if (obj && Array.isArray(obj) && String(arg1) !== 'undefined') {
            const newArray = obj.map((el) => {
                return el[arg1];
            });
            return newArray.toString();
        }
        else {
            if (Array.isArray(obj)) {
                return obj.toString();
            }
            else {
                return obj;
            }
        }
    }
}
/** @nocollapse */ /** @nocollapse */ SclUtilFilterService.ɵfac = function SclUtilFilterService_Factory(t) { return new (t || SclUtilFilterService)(); };
/** @nocollapse */ /** @nocollapse */ SclUtilFilterService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclUtilFilterService, factory: SclUtilFilterService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclUtilFilterService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return []; }, null);
})();

// @ts-ignore
class SclDatatableResponsiveService {
    constructor() { }
    responsiveInit(fieldName) {
        this.checkColHidden(fieldName);
        // this.expandHandler(fieldName);
    }
    changeMediaScreen(fieldName) {
        const doc = document.getElementById(fieldName).childNodes[1];
        doc.childNodes.forEach((val, i) => {
            if (doc.childNodes[i].className === 'child') {
                const el = doc.childNodes[i];
                el.parentNode.removeChild(el);
                //   doc.childNodes[i].remove();
            }
        });
        doc.childNodes.forEach((val, i) => {
            if (doc.childNodes[i].classList.contains('parent')) {
                doc.childNodes[i].classList.remove('parent');
            }
        });
    }
    columnWidthHandler(device) {
        // if(scope.fieldName){
        //   const componentElement = document.getElementById(scope.fieldName);
        //   let maxWidth = this.getMaxWidth(device);
        //   let tWidth = window.innerWidth - 42;
        //   if (componentElement && componentElement.rows && componentElement.rows[0] && componentElement.rows[0]) {
        //     let tds = componentElement.rows[0].children;
        //     let maxColWidth = 0;
        //     // for(let i in tds){
        //     for(let i=0; i<tds.length; i++){
        //       let td = tds[i];
        //       if(window.getComputedStyle(td,null).getPropertyValue('display') != "none"){
        //         let regex = new RegExp("dt-"+device+"-(\\d)", "g");
        //         let wr = parseInt(regex.exec(td.className)[1]);
        //         let cw = (wr/maxWidth)*100;
        //
        //         // td.style.width = cw+"%";
        //       }
        //     }
        //   }
        // }else{
        //   logger.error("fieldName is not set.");
        // }
    }
    getMaxWidth(device) {
        // let maxColWidth = 0;
        // if(scope.fieldName){
        //   const componentElement = document.getElementById(scope.fieldName);
        //
        //   if (componentElement && componentElement.rows && componentElement.rows[1] && componentElement.rows[1]) {
        //     let tds = componentElement.rows[1].children;
        //     // for(let i in tds){
        //     for(let i=0; i<tds.length; i++){
        //       let td = tds[i];
        //       if(window.getComputedStyle(td,null).getPropertyValue('display') != "none"){
        //         let regex = new RegExp("dt-"+device+"-(\\d)", "g");
        //         maxColWidth += parseInt(regex.exec(td.className)[1]);
        //       }
        //     }
        //   }
        // }else{
        //   logger.error("fieldName is not set.");
        // }
        // return maxColWidth;
    }
    calculateWidth(fieldName) {
        if (fieldName) {
            const componentElement = document.getElementById(fieldName);
            if (componentElement && componentElement.rows && componentElement.rows[1] && componentElement.rows[1]) {
                const ths = componentElement.rows[0].children;
                const tblWidth = window.innerWidth - 168;
                let maxWidth = 0;
                for (let i = 0; i < ths.length; i++) {
                    const th = ths[i];
                    if (this.isVisible(th)) {
                        let width = 1;
                        const colW = th.className.match(/colWidth-([0-9])/);
                        if (colW) {
                            width = colW.pop();
                        }
                        maxWidth += (width * 1);
                    }
                }
                for (let i = 0; i < ths.length; i++) {
                    const th = ths[i];
                    if (this.isVisible(th)) {
                        let width = 1;
                        const colW = th.className.match(/colWidth-([0-9])/);
                        if (colW) {
                            width = colW.pop();
                        }
                        const colWidth = (width / maxWidth) * tblWidth;
                        /*th.style.width = ( parseInt(window.getComputedStyle(th, null)
                          .getPropertyValue('width')) + 1) + 'px';*/
                        th.style.width = colWidth + 'px';
                    }
                }
            }
        }
        else {
            console.error('fieldName is not set.');
        }
    }
    isVisible(e) {
        return !!(e.offsetWidth || e.offsetHeight || e.getClientRects().length);
    }
    expandHandler(fieldName) {
        if (fieldName) {
            const componentElement = document.getElementById(fieldName);
            if (componentElement) {
                const tds = componentElement.querySelectorAll('.dataTable.hasHidden tbody td:first-child .handler');
                for (let i = 0; i < tds.length; i++) {
                    const td = tds[i];
                    td.onclick = (e) => {
                        e.stopPropagation();
                        if (document.getElementById(fieldName).classList.contains('hasHidden')) {
                            const headings = document.querySelector('#' + fieldName + ' thead tr').children;
                            const newElement = document.createElement('tr');
                            newElement.className = 'child';
                            const newElementTd = document.createElement('td');
                            let tr = td.parentNode.parentNode;
                            let tbl = tr.parentNode;
                            let tds_child = tr.children;
                            let ul = document.createElement('ul');
                            ul.className = 'dtr-details';
                            if (!tr.classList.contains('scl-dt-parent')) {
                                tr.classList.add('scl-dt-parent');
                                let colSize = 0;
                                // for(let k in tds_child){
                                for (let k = 0; k < tds_child.length; k++) {
                                    let td = tds_child[k];
                                    if (window.getComputedStyle(td, null).getPropertyValue('display') === 'none') {
                                        let li = document.createElement('li');
                                        let spanTitle = document.createElement('span');
                                        let spanData = document.createElement('span');
                                        spanTitle.classList.add('dtr-title');
                                        spanTitle.innerHTML = headings[k].innerHTML;
                                        li.appendChild(spanTitle);
                                        spanData.classList.add('dtr-data');
                                        spanData.innerHTML = td.innerHTML;
                                        li.appendChild(spanData);
                                        ul.appendChild(li);
                                    }
                                    else {
                                        colSize++;
                                    }
                                    // j++;
                                }
                                newElementTd.colSpan = colSize;
                                newElementTd.appendChild(ul);
                                newElement.appendChild(newElementTd);
                                tbl.insertBefore(newElement, tr.nextSibling);
                            }
                            else {
                                tr.classList.remove('scl-dt-parent');
                                let el = tr.nextSibling;
                                el.parentNode.removeChild(el);
                            }
                        }
                        e.preventDefault();
                    };
                }
            }
        }
        else {
            console.log('fieldName is not set.');
        }
    }
    checkColHidden(fieldName) {
        if (fieldName) {
            const componentElement = document.getElementById(fieldName);
            if (componentElement && componentElement.rows && componentElement.rows[0] && componentElement.rows[0]) {
                let tds = componentElement.rows[0].children;
                let foundHidden = false;
                // for(let i in tds){
                for (let i = 0; i < tds.length; i++) {
                    let td = tds[i];
                    if (window.getComputedStyle(td, null).getPropertyValue('display') === 'none') {
                        foundHidden = true;
                        document.getElementById(fieldName).classList.add('hasHidden');
                        break;
                    }
                }
                if (foundHidden) {
                    document.getElementById(fieldName).classList.add('hasHidden');
                }
                else {
                    document.getElementById(fieldName).classList.remove('hasHidden');
                }
                // looking for last-child th
                let isLastElm = false;
                for (let i = tds.length - 1; i >= 0; i--) {
                    if (window.getComputedStyle(tds[i], null).getPropertyValue('display') === 'none') {
                        tds[i].classList.remove('last-child-th');
                    }
                    else {
                        if (!isLastElm) {
                            tds[i].classList.add('last-child-th');
                            isLastElm = true;
                        }
                        else {
                            tds[i].classList.remove('last-child-th');
                        }
                    }
                }
            }
        }
        else {
            console.error('fieldName is not set.');
        }
    }
}
/** @nocollapse */ /** @nocollapse */ SclDatatableResponsiveService.ɵfac = function SclDatatableResponsiveService_Factory(t) { return new (t || SclDatatableResponsiveService)(); };
/** @nocollapse */ /** @nocollapse */ SclDatatableResponsiveService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclDatatableResponsiveService, factory: SclDatatableResponsiveService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclDatatableResponsiveService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return []; }, null);
})();

class SclDatatableResizeService {
    constructor() {
    }
    initResize(excludeResizeCol) {
        let ths = document.querySelectorAll('.dataTable th');
        let tbl = document.querySelector('.dataTable');
        let startOffset = 0;
        let nextOffset = 0;
        let totWidth = 0;
        let currWidth = 0;
        let nextWidth = 0;
        let thElm;
        const padding = 56;
        let mDown = 0;
        let mDrag = 0;
        let minWidth = 20;
        for (let i = 0; i < ths.length; i++) {
            let th = ths[i];
            let tempSpan = document.createElement('span');
            tempSpan.style.height = '100%';
            tempSpan.style.width = '9px';
            tempSpan.style.padding = '22px 0';
            tempSpan.style.position = 'absolute';
            tempSpan.style.right = '-5px';
            tempSpan.style.top = '-6px';
            tempSpan.style.zIndex = '9999';
            th.appendChild(tempSpan);
            if (excludeResizeCol.indexOf(i) === -1) {
                tempSpan.style.cursor = 'col-resize';
                tempSpan.addEventListener('mousedown', (e) => {
                    tbl.addClass('sc-dt-mousedown');
                    thElm = th;
                    const thElmNexSibling = this.nextDisplaySibling(thElm);
                    totWidth = parseFloat(thElm.style.width) + parseFloat(thElmNexSibling.style.width);
                    th.classList.add('disable');
                    startOffset = Number(window.getComputedStyle(th, null).getPropertyValue('width')) - e.pageX;
                    const nextSiblingTh = this.nextDisplaySibling(th);
                    nextOffset = Number(window.getComputedStyle(nextSiblingTh, null).getPropertyValue('width')) + e.pageX;
                    mDown = e.pageX;
                    pauseEvent(e);
                });
            }
            else {
                tempSpan.style.cursor = 'pointer';
            }
        }
        document.addEventListener('mousemove', (e) => {
            if (thElm) {
                pauseEvent(e);
                const thElmNexSibling = this.nextDisplaySibling(thElm);
                if (nextOffset - e.pageX > 0 && startOffset + e.pageX > 0 && Math.abs(mDown - e.pageX) >= 1) {
                    currWidth = (startOffset + e.pageX);
                    // nextWidth = (nextOffset - e.pageX);
                    nextWidth = totWidth - currWidth;
                    if (currWidth < minWidth) {
                        currWidth = minWidth;
                        nextWidth = (totWidth - minWidth);
                    }
                    else if (nextWidth < minWidth) {
                        currWidth = (totWidth - minWidth);
                        nextWidth = minWidth;
                    }
                    thElm.style.width = currWidth + 'px';
                    thElmNexSibling.style.width = nextWidth + 'px';
                }
            }
        });
        const pauseEvent = (e) => {
            if (e.stopPropagation) {
                e.stopPropagation();
            }
            if (e.preventDefault) {
                e.preventDefault();
            }
            e.cancelBubble = true;
            e.returnValue = false;
            return false;
        };
        document.addEventListener('mouseup', (e) => {
            if (thElm) {
                thElm.classList.remove('disable');
            }
            if (currWidth < minWidth) {
                currWidth = minWidth;
                nextWidth = (totWidth - minWidth);
            }
            else if (nextWidth < minWidth) {
                currWidth = (totWidth - minWidth);
                nextWidth = minWidth;
            }
            if (thElm) {
                thElm.style.width = currWidth + 'px';
                thElm.nextSibling.style.width = nextWidth + 'px';
            }
            thElm = undefined;
            mDown = 0;
            tbl.classList.remove('sc-dt-mousedown');
        });
    }
    nextDisplaySibling(elem) {
        let ns = elem.nextSibling;
        while (ns) {
            if (window.getComputedStyle(ns, null).getPropertyValue('display') !== 'none') {
                break;
            }
            ns = ns.nextSibling;
        }
        return ns;
    }
    calculateWidth(fieldName) {
        const componentElement = document.getElementById(fieldName);
        if (componentElement && componentElement.rows && componentElement.rows[1] && componentElement.rows[1]) {
            const ths = document.getElementById(fieldName).rows[0].children;
            for (let i = 0; i < ths.length; i++) {
                const th = ths[i];
                th.style.width = (Number(window.getComputedStyle(th, null).getPropertyValue('width')) + 1) + 'px';
            }
        }
    }
}
/** @nocollapse */ /** @nocollapse */ SclDatatableResizeService.ɵfac = function SclDatatableResizeService_Factory(t) { return new (t || SclDatatableResizeService)(); };
/** @nocollapse */ /** @nocollapse */ SclDatatableResizeService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclDatatableResizeService, factory: SclDatatableResizeService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclDatatableResizeService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return []; }, null);
})();

class UtilDocumentIdService {
    constructor() {
        this.uuidLength = 36;
    }
    generateId() {
        // use a v1 id. This includes the device hardware in the generation, so there is more chance of uniqueness
        return uuid.v1();
    }
    isValid(id) {
        // start with an initial test of checking that the length is correct
        return id && id.length === this.uuidLength;
    }
}
/** @nocollapse */ /** @nocollapse */ UtilDocumentIdService.ɵfac = function UtilDocumentIdService_Factory(t) { return new (t || UtilDocumentIdService)(); };
/** @nocollapse */ /** @nocollapse */ UtilDocumentIdService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: UtilDocumentIdService, factory: UtilDocumentIdService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(UtilDocumentIdService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return []; }, null);
})();

class SclUtilDocumentService {
    constructor(utilDocumentIdService) {
        this.utilDocumentIdService = utilDocumentIdService;
    }
    isNew(doc) {
        return !doc.systemHeader || !doc.systemHeader.createdDate;
    }
    getValidId(value) {
        let validValue;
        if (!value) {
            validValue = this.utilDocumentIdService.generateId();
        }
        else {
            validValue = value;
        }
        return validValue;
    }
}
/** @nocollapse */ /** @nocollapse */ SclUtilDocumentService.ɵfac = function SclUtilDocumentService_Factory(t) { return new (t || SclUtilDocumentService)(i0.ɵɵinject(UtilDocumentIdService)); };
/** @nocollapse */ /** @nocollapse */ SclUtilDocumentService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclUtilDocumentService, factory: SclUtilDocumentService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclUtilDocumentService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return [{ type: UtilDocumentIdService }]; }, null);
})();

class SclExportService {
    constructor(httpClient) {
        this.httpClient = httpClient;
        this.format = 'csv';
        this.headers = [];
    }
    export(qry, filename, settings, handlebarsHelpers) {
        var _a;
        const type = this.format;
        filename = filename + '.' + type;
        if (!this.headers) {
            this.headers = [{ title: 'ID', field: 'documentId' }, { title: 'name', field: 'systemHeader.summaryName' }];
        }
        const body = {
            headers: this.headers,
            query: qry,
            fileName: filename,
            type: type,
            size: (settings === null || settings === void 0 ? void 0 : settings.size) || 'A4',
            layout: (settings === null || settings === void 0 ? void 0 : settings.layout) || 'landscape',
            fontSize: (settings === null || settings === void 0 ? void 0 : settings.fontSize) || 10,
            margin: (settings === null || settings === void 0 ? void 0 : settings.margin) || 25,
            headerFontSize: (settings === null || settings === void 0 ? void 0 : settings.headerFontSize) || 8,
            columnSpacing: (settings === null || settings === void 0 ? void 0 : settings.columnSpacing) || 5,
            urlExportMode: (settings === null || settings === void 0 ? void 0 : settings.urlExportMode) || 'linkObject',
            handlebarsHelpers: handlebarsHelpers || [],
            stripHTML: (_a = settings === null || settings === void 0 ? void 0 : settings.stripHTML) !== null && _a !== void 0 ? _a : true
        };
        const options = {
            headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
            responseType: 'blob'
        };
        return this.httpClient.post('/api/searchExport', body, options);
    }
}
/** @nocollapse */ /** @nocollapse */ SclExportService.ɵfac = function SclExportService_Factory(t) { return new (t || SclExportService)(i0.ɵɵinject(i1$1.HttpClient)); };
/** @nocollapse */ /** @nocollapse */ SclExportService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclExportService, factory: SclExportService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclExportService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return [{ type: i1$1.HttpClient }]; }, null);
})();

class SclUtilDateService {
    constructor() { }
    getLocalFromUTCString(UTCTime) {
        // It'll convert the utc -> localtime
        return new Date(UTCTime);
    }
    // LocalTime must be formatted at ISOString i.e 2015-04-08T12:12:12.000Z
    getUTCFromLocalString(localTime) {
        let dateTime = localTime.replace('T', ' ').replace('Z', '');
        dateTime = new Date(dateTime); // create date object
        return dateTime.toISOString();
    }
    parseDate(date, format) {
        return moment(date).format(format);
    }
    isDate(dateObject) {
        if (Object.prototype.toString.call(dateObject) === '[object Date]') {
            if (isNaN(dateObject.getTime()) === false) {
                return true;
            }
        }
        return false;
    }
    isValidDate(date) {
        // An invalid date object returns NaN for getTime() and NaN is the only
        // object not strictly equal to itself.
        return date.getTime() === date.getTime();
    }
    formatDate(time) {
        return moment(time).fromNow();
    }
    getValidDate(value) {
        let validValue;
        if (!value) {
            validValue = new Date();
        }
        else {
            if (this.isValidDate(value)) {
                validValue = value;
            }
            else {
                const date = moment(value, ['DD/MM/YYYY hh:mm a', 'DD/MM/YYYY', 'hh:mm a', moment.ISO_8601], true);
                if (date.isValid()) {
                    validValue = date.toDate();
                }
                else {
                    validValue = new Date();
                }
            }
        }
        return validValue;
    }
    getFormattedDate(dateValue) {
        if (!dateValue) {
            return dateValue;
        }
        let formattedDate;
        let d;
        d = new Date(dateValue);
        const defaultYear = new Date();
        if (d.getFullYear() < 1) {
            d.setYear(defaultYear.getFullYear());
            formattedDate = d;
        }
        else {
            formattedDate = d;
        }
        return formattedDate;
    }
}
/** @nocollapse */ /** @nocollapse */ SclUtilDateService.ɵfac = function SclUtilDateService_Factory(t) { return new (t || SclUtilDateService)(); };
/** @nocollapse */ /** @nocollapse */ SclUtilDateService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclUtilDateService, factory: SclUtilDateService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclUtilDateService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return []; }, null);
})();

class DeviceDetectService {
    constructor() {
    }
    detectBrowser() {
        const ua = window.navigator.userAgent;
        let device;
        let browser;
        let data = {};
        // get device
        if (ua.match(/(iPhone)/)) {
            device = 'iPhone';
        }
        else if (ua.match(/(iPad)/)) {
            device = 'iPad';
        }
        else if (ua.match(/(iPod)/)) {
            device = 'iPod';
        }
        else if (ua.match(/(BlackBerry|BB10)/)) {
            device = 'BlackBerry';
        }
        else if (ua.match(/(IEMobile)/)) {
            device = 'WindowsMobile';
        }
        else if (ua.match(/(Android)/)) {
            device = 'Android';
        }
        else if (ua.match(/(Macintosh)/)) {
            device = 'Macintosh';
        }
        else if (ua.match(/(Windows)/)) {
            device = 'Windows';
        }
        else if (ua.match(/(Linux)/)) {
            device = 'Linux';
        }
        else if (ua.match(/(Opera Mini)/)) {
            device = 'OperaMini';
        }
        else if (ua.match(/(Mobi)/)) {
            device = 'Mobi';
        }
        else if (ua.match(/(IEMobi)/)) {
            device = 'Mobi';
        }
        // get browser
        if (ua.indexOf('OPR') > 0 || ua.indexOf('Opera') > 0) {
            browser = 'Opera';
        }
        else if (ua.indexOf('Chrome') > 0 && ua.indexOf('Edge') === -1) {
            browser = 'Chrome';
        }
        else if (ua.indexOf('Firefox') > 0) {
            browser = 'Firefox';
        }
        else if (ua.indexOf('Safari') > 0 && ua.indexOf('Edge') === -1) {
            browser = 'Safari';
        }
        else if (ua.indexOf('MSIE') > 0) {
            browser = 'IE';
        }
        else if (ua.indexOf('Edge') > 0) {
            browser = 'Edge';
        }
        // create object
        data = {
            device: device,
            browser: browser
        };
        return data;
    }
    detectMobile() {
        const toMatch = [
            /Android/i,
            /webOS/i,
            /iPhone/i,
            /iPad/i,
            /iPod/i,
            /BlackBerry/i,
            /Windows Phone/i
        ];
        return toMatch.some((toMatchItem) => {
            return window.navigator.userAgent.match(toMatchItem);
        });
    }
}
/** @nocollapse */ /** @nocollapse */ DeviceDetectService.ɵfac = function DeviceDetectService_Factory(t) { return new (t || DeviceDetectService)(); };
/** @nocollapse */ /** @nocollapse */ DeviceDetectService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: DeviceDetectService, factory: DeviceDetectService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(DeviceDetectService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return []; }, null);
})();

class SclCsvParserService {
    constructor() {
    }
    getBaby() {
        const Baby = {
            BAD_DELIMITERS: undefined, BYTE_ORDER_MARK: undefined,
            DefaultDelimiter: undefined, RECORD_SEP: undefined, UNIT_SEP: undefined
        };
        Baby.BAD_DELIMITERS = ['\r', '\n', '"', Baby.BYTE_ORDER_MARK];
        return Baby;
    }
    Parser(config) {
        const Baby = this.getBaby();
        // Unpack the config object
        config = config || {};
        let delim = config.delimiter;
        let newline = config.newline;
        let comments = config.comments;
        const step = config.step;
        const preview = config.preview;
        const fastMode = config.fastMode;
        // Delimiter must be valid
        if (typeof delim !== 'string'
            || delim.length !== 1
            || Baby.BAD_DELIMITERS.indexOf(delim) > -1) {
            delim = ',';
        }
        // Comment character must be valid
        if (comments === delim) {
            throw new Error('Comment character same as delimiter');
        }
        else if (comments === true) {
            comments = '#';
        }
        else if (typeof comments !== 'string'
            || Baby.BAD_DELIMITERS.indexOf(comments) > -1) {
            comments = false;
        }
        // Newline must be valid: \r, \n, or \r\n
        if (newline !== '\n' && newline !== '\r' && newline !== '\r\n') {
            newline = '\n';
        }
        // We're gonna need these at the Parser scope
        let cursor = 0;
        let aborted = false;
        function parse(input) {
            // For some reason, in Chrome, this speeds things up (!?)
            if (typeof input !== 'string') {
                throw new Error('Input must be a string');
            }
            // We don't need to compute some of these every time parse() is called,
            // but having them in a more local scope seems to perform better
            const inputLen = input.length;
            const delimLen = delim.length;
            const newlineLen = newline.length;
            const commentsLen = comments.length;
            const stepIsFunction = typeof step === 'function';
            // Establish starting state
            cursor = 0;
            let data = [];
            let errors = [];
            let row = [];
            if (!input) {
                return returnable();
            }
            if (fastMode) {
                // Fast mode assumes there are no quoted fields in the input
                const rows = input.split(newline);
                for (let i = 0; i < rows.length; i++) {
                    if (comments && rows[i].substr(0, commentsLen) === comments) {
                        continue;
                    }
                    if (stepIsFunction) {
                        data = [rows[i].split(delim)];
                        doStep();
                        if (aborted) {
                            return returnable();
                        }
                    }
                    else {
                        data.push(rows[i].split(delim));
                    }
                    if (preview && i >= preview) {
                        data = data.slice(0, preview);
                        return returnable(true);
                    }
                }
                return this.returnable();
            }
            let nextDelim = input.indexOf(delim, cursor);
            let nextNewline = input.indexOf(newline, cursor);
            // Parser loop
            for (;;) {
                // Field has opening quote
                if (input[cursor] === '"') {
                    // Start our search for the closing quote where the cursor is
                    let quoteSearch = cursor;
                    // Skip the opening quote
                    cursor++;
                    for (;;) {
                        // Find closing quote
                        quoteSearch = input.indexOf('"', quoteSearch + 1);
                        if (quoteSearch === -1) {
                            // No closing quote... what a pity
                            errors.push({
                                type: 'Quotes',
                                code: 'MissingQuotes',
                                message: 'Quoted field unterminated',
                                row: data.length,
                                index: cursor
                            });
                            return finish();
                        }
                        if (quoteSearch === inputLen - 1) {
                            // Closing quote at EOF
                            row.push(input.substring(cursor, quoteSearch).replace(/""/g, '"'));
                            data.push(row);
                            if (stepIsFunction) {
                                doStep();
                            }
                            return this.returnable();
                        }
                        // If this quote is escaped, it's part of the data; skip it
                        // tslint:disable-next-line:triple-equals
                        if (input[quoteSearch + 1] == '"') {
                            quoteSearch++;
                            continue;
                        }
                        if (input[quoteSearch + 1] === delim) {
                            // Closing quote followed by delimiter
                            row.push(input.substring(cursor, quoteSearch).replace(/""/g, '"'));
                            cursor = quoteSearch + 1 + delimLen;
                            nextDelim = input.indexOf(delim, cursor);
                            nextNewline = input.indexOf(newline, cursor);
                            break;
                        }
                        if (input.substr(quoteSearch + 1, newlineLen) === newline) {
                            // Closing quote followed by newline
                            row.push(input.substring(cursor, quoteSearch).replace(/""/g, '"'));
                            saveRow(quoteSearch + 1 + newlineLen);
                            nextDelim = input.indexOf(delim, cursor); // because we may have skipped the nextDelim in the quoted field
                            if (stepIsFunction) {
                                doStep();
                                if (aborted) {
                                    return this.returnable();
                                }
                            }
                            if (preview && data.length >= preview) {
                                return returnable(true);
                            }
                            break;
                        }
                    }
                    continue;
                }
                // Comment found at start of new line
                if (comments && row.length === 0 && input.substr(cursor, commentsLen) === comments) {
                    if (nextNewline === -1) { // Comment ends at EOF
                        return this.returnable();
                    }
                    cursor = nextNewline + newlineLen;
                    nextNewline = input.indexOf(newline, cursor);
                    nextDelim = input.indexOf(delim, cursor);
                    continue;
                }
                // Next delimiter comes before next newline, so we've reached end of field
                if (nextDelim !== -1 && (nextDelim < nextNewline || nextNewline === -1)) {
                    row.push(input.substring(cursor, nextDelim));
                    cursor = nextDelim + delimLen;
                    nextDelim = input.indexOf(delim, cursor);
                    continue;
                }
                // End of row
                if (nextNewline !== -1) {
                    row.push(input.substring(cursor, nextNewline));
                    saveRow(nextNewline + newlineLen);
                    if (stepIsFunction) {
                        doStep();
                        if (aborted) {
                            return this.returnable();
                        }
                    }
                    if (preview && data.length >= preview) {
                        return returnable(true);
                    }
                    continue;
                }
                break;
            }
            return finish();
            // Appends the remaining input from cursor to the end into
            // row, saves the row, calls step, and returns the results.
            function finish() {
                row.push(input.substr(cursor));
                data.push(row);
                cursor = inputLen; // important in case parsing is paused
                if (stepIsFunction) {
                    doStep();
                }
                return returnable();
            }
            // Appends the current row to the results. It sets the cursor
            // to newCursor and finds the nextNewline. The caller should
            // take care to execute user's step function and check for
            // preview and end parsing if necessary.
            function saveRow(newCursor) {
                data.push(row);
                row = [];
                cursor = newCursor;
                nextNewline = input.indexOf(newline, cursor);
            }
            // Returns an object with the results, errors, and meta.
            function returnable(stopped) {
                return {
                    data: data,
                    errors: errors,
                    meta: {
                        delimiter: delim,
                        linebreak: newline,
                        aborted: aborted,
                        truncated: !!stopped
                    }
                };
            }
            // Executes the user's step function and resets data & errors.
            function doStep() {
                step(returnable());
                data = [], errors = [];
            }
        }
        // Sets the abort flag
        function abort() {
            aborted = true;
        }
        // Gets the cursor position
        function getCharIndex() {
            return cursor;
        }
    }
}
/** @nocollapse */ /** @nocollapse */ SclCsvParserService.ɵfac = function SclCsvParserService_Factory(t) { return new (t || SclCsvParserService)(); };
/** @nocollapse */ /** @nocollapse */ SclCsvParserService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclCsvParserService, factory: SclCsvParserService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclCsvParserService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return []; }, null);
})();

const sclCsvParser$1 = new SclCsvParserService();
class SclCsvParserHandlerService {
    constructor() {
    }
    ParserHandle(_CONFIG) {
        const Baby = sclCsvParser$1.getBaby();
        // One goal is to minimize the use of regular expressions...
        const FLOAT = /^\s*-?(\d*\.?\d+|\d+\.?\d*)(e[-+]?\d+)?\s*$/i;
        const self = this;
        let stepCounter = 0; // Number of times step was called (number of rows parsed)
        let _INPUT = null; // The input being parsed
        let parser = null; // The core parser being used
        let paused = false; // Whether we are paused or not
        let delimiterError = false; // Temporary state between delimiter detection and processing results
        const fields = []; // Fields are from the header row of the input, if there is one
        let _RESULTS = {
            data: [],
            errors: [],
            meta: {
                fields: undefined,
                delimiter: undefined
            }
        };
        const isFunction = (func) => {
            return typeof func === 'function';
        };
        const copy = (obj) => {
            if (typeof obj !== 'object') {
                return obj;
            }
            const cpy = obj instanceof Array ? [] : {};
            for (const key in obj) {
                if (obj[key]) {
                    cpy[key] = copy(obj[key]);
                }
            }
            return cpy;
        };
        if (isFunction(_CONFIG.step)) {
            const userStep = _CONFIG.step;
            _CONFIG.step = (results) => {
                _RESULTS = results;
                if (needsHeaderRow()) {
                    processResults();
                }
                else {
                    processResults();
                    // It's possbile that this line was empty and there's no row here after all
                    if (_RESULTS.data.length === 0) {
                        return;
                    }
                    stepCounter += results.data.length;
                    if (_CONFIG.preview && stepCounter > _CONFIG.preview) {
                        parser.abort();
                    }
                    else {
                        userStep(_RESULTS, self);
                    }
                }
            };
        }
        function parse(input) {
            if (!_CONFIG.newline) {
                _CONFIG.newline = guessLineEndings(input);
            }
            delimiterError = false;
            if (!_CONFIG.delimiter) {
                const delimGuess = guessDelimiter(input);
                if (delimGuess.successful) {
                    _CONFIG.delimiter = delimGuess.bestDelimiter;
                }
                else {
                    delimiterError = true; // add error after parsing (otherwise it would be overwritten)
                    // @ts-ignore
                    _CONFIG.delimiter = Baby.DefaultDelimiter;
                }
                _RESULTS.meta.delimiter = _CONFIG.delimiter;
            }
            const parserConfig = copy(_CONFIG);
            if (_CONFIG.preview && _CONFIG.header) {
                parserConfig.preview++;
            } // to compensate for header row
            _INPUT = input;
            parser = new sclCsvParser$1.Parser(parserConfig);
            _RESULTS = parser.parse(_INPUT);
            processResults();
            // @ts-ignore
            if (isFunction(_CONFIG.complete) && !paused && (!self.streamer || self.streamer.finished())) {
                _CONFIG.complete(_RESULTS);
            }
            return paused ? { meta: { paused: true } } : (_RESULTS || { meta: { paused: false } });
        }
        function pause() {
            paused = true;
            parser.abort();
            _INPUT = _INPUT.substr(parser.getCharIndex());
        }
        function resume() {
            paused = false;
            parser = new sclCsvParser$1.Parser(_CONFIG);
            parser.parse(_INPUT);
            if (!paused) {
                // @ts-ignore
                if (self.streamer && !self.streamer.finished()) {
                    // @ts-ignore
                    self.streamer.resume();
                }
                else if (isFunction(_CONFIG.complete)) {
                    _CONFIG.complete(_RESULTS);
                }
            }
        }
        function abort() {
            parser.abort();
            if (isFunction(_CONFIG.complete)) {
                _CONFIG.complete(_RESULTS);
            }
            _INPUT = '';
        }
        function processResults() {
            if (_RESULTS && delimiterError) {
                // tslint:disable-next-line:max-line-length
                addError('Delimiter', 'UndetectableDelimiter', 'Unable to auto-detect delimiting character; defaulted to \'' + Baby.DefaultDelimiter + '\'');
                delimiterError = false;
            }
            if (_CONFIG.skipEmptyLines) {
                for (let i = 0; i < _RESULTS.data.length; i++) {
                    if (_RESULTS.data[i].length === 1 && _RESULTS.data[i][0] === '') {
                        _RESULTS.data.splice(i--, 1);
                    }
                }
            }
            if (needsHeaderRow()) {
                fillHeaderFields();
            }
            return applyHeaderAndDynamicTyping();
        }
        function needsHeaderRow() {
            return _CONFIG.header && fields.length === 0;
        }
        function fillHeaderFields() {
            if (!_RESULTS) {
                return;
            }
            for (let i = 0; needsHeaderRow() && i < _RESULTS.data.length; i++) {
                for (let j = 0; j < _RESULTS.data[i].length; j++) {
                    fields.push(_RESULTS.data[i][j]);
                }
            }
            _RESULTS.data.splice(0, 1);
        }
        function applyHeaderAndDynamicTyping() {
            if (!_RESULTS || (!_CONFIG.header && !_CONFIG.dynamicTyping)) {
                return _RESULTS;
            }
            for (let i = 0; i < _RESULTS.data.length; i++) {
                const row = {};
                for (let j = 0; j < _RESULTS.data[i].length; j++) {
                    if (_CONFIG.dynamicTyping) {
                        const value = _RESULTS.data[i][j];
                        if (value === 'true' || value === 'TRUE') {
                            _RESULTS.data[i][j] = true;
                        }
                        else if (value === 'false' || value === 'FALSE') {
                            _RESULTS.data[i][j] = false;
                        }
                        else {
                            _RESULTS.data[i][j] = tryParseFloat(value);
                        }
                    }
                    if (_CONFIG.header) {
                        if (j >= fields.length) {
                            // @ts-ignore
                            if (!row.__parsed_extra) {
                                // @ts-ignore
                                row.__parsed_extra = [];
                            }
                            // @ts-ignore
                            row.__parsed_extra.push(_RESULTS.data[i][j]);
                        }
                        else {
                            row[fields[j]] = _RESULTS.data[i][j];
                        }
                    }
                    if (_CONFIG.header) {
                        _RESULTS.data[i] = row;
                        if (j > fields.length) {
                            addError('FieldMismatch', 'TooManyFields', 'Too many fields: expected ' + fields.length + ' fields but parsed ' + j, i);
                        }
                        else if (j < fields.length) {
                            addError('FieldMismatch', 'TooFewFields', 'Too few fields: expected ' + fields.length + ' fields but parsed ' + j, i);
                        }
                    }
                }
            }
            if (_CONFIG.header && _RESULTS.meta) {
                _RESULTS.meta.fields = fields;
            }
            return _RESULTS;
        }
        function guessDelimiter(input) {
            const delimChoices = [',', '\t', '|', ';', Baby.RECORD_SEP, Baby.UNIT_SEP];
            let bestDelim;
            let bestDelta;
            let fieldCountPrevRow;
            for (let i = 0; i < delimChoices.length; i++) {
                const delim = delimChoices[i];
                let delta = 0;
                let avgFieldCount = 0;
                fieldCountPrevRow = undefined;
                const preview = new sclCsvParser$1.Parser({
                    delimiter: delim,
                    preview: 10
                }).parse(input);
                for (let j = 0; j < preview.data.length; j++) {
                    const fieldCount = preview.data[j].length;
                    avgFieldCount += fieldCount;
                    if (typeof fieldCountPrevRow === 'undefined') {
                        fieldCountPrevRow = fieldCount;
                        continue;
                    }
                    else if (fieldCount > 1) {
                        delta += Math.abs(fieldCount - fieldCountPrevRow);
                        fieldCountPrevRow = fieldCount;
                    }
                }
                avgFieldCount /= preview.data.length;
                if ((typeof bestDelta === 'undefined' || delta < bestDelta)
                    && avgFieldCount > 1.99) {
                    bestDelta = delta;
                    bestDelim = delim;
                }
            }
            _CONFIG.delimiter = bestDelim;
            return {
                successful: !!bestDelim,
                bestDelimiter: bestDelim
            };
        }
        function guessLineEndings(input) {
            input = input.substr(0, 1024 * 1024); // max length 1 MB
            const r = input.split('\r');
            if (r.length === 1) {
                return '\n';
            }
            let numWithN = 0;
            for (let i = 0; i < r.length; i++) {
                if (r[i][0] === '\n') {
                    numWithN++;
                }
            }
            return numWithN >= r.length / 2 ? '\r\n' : '\r';
        }
        function tryParseFloat(val) {
            const isNumber = FLOAT.test(val);
            return isNumber ? parseFloat(val) : val;
        }
        function addError(type, code, msg, row) {
            _RESULTS.errors.push({
                type: type,
                code: code,
                message: msg,
                row: row
            });
        }
    }
}
/** @nocollapse */ /** @nocollapse */ SclCsvParserHandlerService.ɵfac = function SclCsvParserHandlerService_Factory(t) { return new (t || SclCsvParserHandlerService)(); };
/** @nocollapse */ /** @nocollapse */ SclCsvParserHandlerService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclCsvParserHandlerService, factory: SclCsvParserHandlerService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclCsvParserHandlerService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return []; }, null);
})();

const sclCsvParserHandler = new SclCsvParserHandlerService();
const sclCsvParser = new SclCsvParserService();
class SclUtilImporterService {
    constructor(formbirdInjectorService) {
        this.formbirdInjectorService = formbirdInjectorService;
        this.DEFAULTS = {
            delimiter: '',
            newline: '',
            header: false,
            dynamicTyping: false,
            preview: 0,
            step: undefined,
            comments: false,
            complete: undefined,
            skipEmptyLines: false,
            fastMode: false
        };
        this.reIsHostCtor = /^\[object .+?Constructor\]$/;
        this.reIsUint = /^\d+$/;
        this.MAX_SAFE_INTEGER = 9007199254740991;
        this.reIsNative = RegExp('^' +
            Function.prototype.toString.call(Object.prototype.hasOwnProperty)
                .replace(/[\\^$.*+?()[\]{}|]/g, '\\$&')
                .replace(/Object.prototype.hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g, '$1.*?') + '$');
        this.arrayTag = '[object Array]';
        this.rePropName = /[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\n\\]|\\.)*?)\2)\]/g;
        this.reEscapeChar = /\\(\\)?/g;
        this.reIsDeepProp = /\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\n\\]|\\.)*?\1)\]/;
        this.reIsPlainProp = /^\w*$/;
        this.docIdMatch = '';
        this.docIdNoMatch = '';
        this.notificationService = this.formbirdInjectorService.get('NotificationService');
        this.dataService = this.formbirdInjectorService.get('DataService');
    }
    isObjectLike(value) {
        return !!value && typeof value === 'object';
    }
    isLength(value) {
        return typeof value === 'number' && value > -1 && value % 1 === 0 && value <= this.MAX_SAFE_INTEGER;
    }
    isFunction(func) {
        return typeof func === 'function';
    }
    isNative(value) {
        const fnToString = Function.prototype.toString;
        if (value == null) {
            return false;
        }
        if (this.isFunction(value)) {
            return this.reIsNative.test(fnToString.call(value));
        }
        return this.isObjectLike(value) && this.reIsHostCtor.test(value);
    }
    getNative(object, key) {
        const value = object == null ? undefined : object[key];
        return this.isNative(value) ? value : undefined;
    }
    isArray(value) {
        const nativeIsArray = this.getNative(Array, 'isArray');
        const objToString = Object.prototype.toString;
        if (!value) {
            return nativeIsArray;
        }
        return this.isObjectLike(value) && this.isLength(value.length) && objToString.call(value) === this.arrayTag;
    }
    isObject(value) {
        const type = typeof value;
        return !!value && (type === 'object' || type === 'function');
    }
    isIndex(value, length) {
        value = (typeof value === 'number' || this.reIsUint.test(value)) ? +value : -1;
        length = length == null ? this.MAX_SAFE_INTEGER : length;
        return value > -1 && value % 1 === 0 && value < length;
    }
    baseToString(value) {
        return value == null ? '' : (value + '');
    }
    toPath(value) {
        if (this.isArray(value)) {
            return value;
        }
        const result = [];
        // @ts-ignore
        // tslint:disable-next-line:variable-name
        this.baseToString(value).replace(this.rePropName, function (match, number, quote, string) {
            result.push(quote ? string.replace(this.reEscapeChar, '$1') : (number || match));
        });
        return result;
    }
    toObject(value) {
        return this.isObject(value) ? value : Object(value);
    }
    isKey(value, object) {
        const type = typeof value;
        if ((type === 'string' && this.reIsPlainProp.test(value)) || type === 'number') {
            return true;
        }
        if (this.isArray(value)) {
            return false;
        }
        const result = !this.reIsDeepProp.test(value);
        return result || (object != null && value in this.toObject(object));
    }
    set(object, path, value) {
        if (object == null) {
            return object;
        }
        if (path === 'date' && value === '') {
            // value = undefined;
        }
        const pathKey = (path + '');
        path = (object[pathKey] != null || this.isKey(path, object)) ? [pathKey] : this.toPath(path);
        let index = -1;
        const length = path.length;
        const lastIndex = length - 1;
        let nested = object;
        while (nested != null && ++index < length) {
            const key = path[index];
            if (this.isObject(nested)) {
                if (index === lastIndex) {
                    nested[key] = value;
                }
                else if (nested[key] == null) {
                    nested[key] = this.isIndex(path[index + 1]) ? [] : {};
                }
            }
            nested = nested[key];
        }
        return object;
    }
    validateColumnHeader(dataObject, columnsLabels) {
        let isHeader = true;
        if (dataObject.length > 0 && columnsLabels && dataObject[0].length === columnsLabels.length) {
            for (let i = 0; i < dataObject[0].length; i++) {
                if (dataObject[0][i] !== columnsLabels[i]) {
                    isHeader = false;
                    break;
                }
            }
        }
        else {
            isHeader = false;
        }
        return isHeader;
    }
    processDocumentIdMatch(newDoc, oldDoc, docIdMatch) {
        if (docIdMatch === 'merge') {
            for (const key in newDoc) {
                if (key !== 'documentId' && key !== 'systemHeader' && newDoc.hasOwnProperty(key)) {
                    if (oldDoc[key] instanceof Array && newDoc[key] instanceof Array) {
                        let dataObj = [];
                        dataObj = dataObj.concat(oldDoc[key]);
                        dataObj = dataObj.concat(newDoc[key]);
                        oldDoc[key] = dataObj;
                    }
                    else if (oldDoc[key] instanceof Object && newDoc[key] instanceof Object) {
                        const dataObj = {};
                        for (const attr in oldDoc[key]) {
                            if (oldDoc[key][attr]) {
                                dataObj[attr] = oldDoc[key][attr];
                            }
                        }
                        for (const attr in newDoc[key]) {
                            if (newDoc[key][attr]) {
                                dataObj[attr] = newDoc[key][attr];
                            }
                        }
                        oldDoc[key] = dataObj;
                    }
                    else {
                        oldDoc[key] = newDoc[key];
                    }
                }
            }
            return oldDoc;
        }
        else if (docIdMatch === 'replace') {
            newDoc.systemHeader = oldDoc.systemHeader;
            return Object.assign(Object.assign({}, oldDoc), newDoc);
        }
        else {
            return newDoc;
        }
    }
    processDocumentIdNoMatch(newDoc, docIdNoMatch) {
        if (docIdNoMatch === 'add') {
            return newDoc;
        }
        else if (this.docIdNoMatch === 'error') {
            newDoc.errMsg = 'documentId \'' + newDoc.documentId + '\' is not in database and adding is not enabled';
            return newDoc;
        }
    }
    setComponentScopeData(compScope) {
        this.componentScope = compScope;
    }
    getComponentScope() {
        return this.componentScope;
    }
    processInsertDocIdNoMatch(document, csvIndex, docIdNoMatch) {
        const self = this;
        if (docIdNoMatch === 'add') {
            this.dataService.insert(document).then(function successFunc(data) {
                console.log(data, 'return');
                self.insertSuccess(data, csvIndex);
            }, function errorFunc(err) {
                self.insertFail(err, csvIndex);
            });
        }
        else if (docIdNoMatch === 'error') {
            const dataItemsMatTableData = this.componentScope.cloneDeepFunction(this.componentScope.itemsMatTableData);
            dataItemsMatTableData.map((element, index) => {
                if (csvIndex === index) {
                    document.documentId = element['colData' + (this.componentScope.getIndexOfDocumentIdInColumnLabels() + 1)];
                }
            });
            document.errMsg = 'documentId \'' + document.documentId + '\' is not in database and adding is not enabled';
            this.insertFail(document.errMsg, csvIndex);
        }
    }
    insertSuccess(data, csvIdx, actionName) {
        actionName = (actionName ? actionName : 'Imported');
        this.notificationService.success(data.documentId + ' ' + actionName + '!');
        // logger.info(data.documentId + ' ' + actionName + '!');
        const dataItems = this.componentScope.cloneDeepFunction(this.componentScope.items);
        const dataItemsMatTableData = this.componentScope.cloneDeepFunction(this.componentScope.itemsMatTableData);
        const csvPrefix = this.componentScope.csvFieldNamePrefix ? this.componentScope.csvFieldNamePrefix : '';
        dataItems.map((element, index) => {
            if (csvIdx === index) {
                element[csvPrefix + '0'] = actionName + ' - ' + data.documentId;
                element.documentId = data.documentId;
                element[csvPrefix + (this.componentScope.getIndexOfDocumentIdInColumnLabels() + 1)] = data.documentId;
            }
        });
        dataItemsMatTableData.map((element, index) => {
            if (csvIdx === index) {
                element.colData0 = actionName + ' - ' + data.documentId;
                element.documentId = data.documentId;
                element['colData' + (this.componentScope.getIndexOfDocumentIdInColumnLabels() + 1)] = data.documentId;
            }
        });
        this.componentScope.tempDocId = data.documentId;
        this.componentScope.documentCreated++;
        this.componentScope.items = dataItems;
        this.componentScope.itemsMatTableData = dataItemsMatTableData;
        this.componentScope.loadItemsToDataTable(this.componentScope.itemsMatTableData);
        this.componentScope.valueChanged(this.componentScope.items);
        this.componentScope.document[this.componentScope.fieldName] = this.componentScope.items;
        this.componentScope.detectChanges();
    }
    insertFail(err, csvIdx, actionName) {
        let errMsg = '';
        if (actionName) {
            actionName = (actionName === 'Updated' ? 'updating' : 'replacing');
            errMsg = 'Failed - ' + err.message.replace('updating', actionName);
        }
        else {
            errMsg = 'Failed - ' + err.message;
        }
        if (!err.message) {
            errMsg = 'Failed - ' + err;
        }
        this.notificationService.error(errMsg);
        const dataItems = this.componentScope.cloneDeepFunction(this.componentScope.items);
        const dataItemsMatTableData = this.componentScope.cloneDeepFunction(this.componentScope.itemsMatTableData);
        const csvPrefix = this.componentScope.csvFieldNamePrefix ? this.componentScope.csvFieldNamePrefix : '';
        dataItems.map((element, index) => {
            if (csvIdx === index) {
                element[csvPrefix + '0'] = errMsg;
            }
        });
        dataItemsMatTableData.map((element, index) => {
            if (csvIdx === index) {
                element.colData0 = errMsg;
            }
        });
        // console.log(csvIdx, dataItems, dataItemsMatTableData);
        this.componentScope.errors++;
        this.componentScope.items = dataItems;
        this.componentScope.itemsMatTableData = dataItemsMatTableData;
        this.componentScope.loadItemsToDataTable(this.componentScope.itemsMatTableData);
        this.componentScope.valueChanged(this.componentScope.items);
        this.componentScope.document[this.componentScope.fieldName] = this.componentScope.items;
        this.componentScope.detectChanges();
    }
    isRowData(rowData) {
        let hasData = false;
        for (let i = 0; i < rowData.length; i++) {
            if (rowData[i] !== '') {
                hasData = true;
                break;
            }
        }
        return hasData;
    }
    isRowDataObj(rowData) {
        let hasData = false;
        for (const key in rowData) {
            if (rowData[key] !== '') {
                hasData = true;
                break;
            }
        }
        return hasData;
    }
    copy(obj) {
        if (typeof obj !== 'object') {
            return obj;
        }
        const cpy = obj instanceof Array ? [] : {};
        for (const key in obj) {
            if (obj[key]) {
                cpy[key] = this.copy(obj[key]);
            }
        }
        return cpy;
    }
    copyAndValidateConfig(origConfig) {
        const Baby = sclCsvParser.getBaby();
        if (typeof origConfig !== 'object') {
            origConfig = {};
        }
        const config = this.copy(origConfig);
        if (typeof config.delimiter !== 'string'
            || config.delimiter.length !== 1
            || Baby.BAD_DELIMITERS.indexOf(config.delimiter) > -1) {
            config.delimiter = this.DEFAULTS.delimiter;
        }
        if (config.newline !== '\n'
            && config.newline !== '\r'
            && config.newline !== '\r\n') {
            config.newline = this.DEFAULTS.newline;
        }
        if (typeof config.header !== 'boolean') {
            config.header = this.DEFAULTS.header;
        }
        if (typeof config.dynamicTyping !== 'boolean') {
            config.dynamicTyping = this.DEFAULTS.dynamicTyping;
        }
        if (typeof config.preview !== 'number') {
            config.preview = this.DEFAULTS.preview;
        }
        if (typeof config.step !== 'function') {
            config.step = this.DEFAULTS.step;
        }
        if (typeof config.complete !== 'function') {
            config.complete = this.DEFAULTS.complete;
        }
        if (typeof config.skipEmptyLines !== 'boolean') {
            config.skipEmptyLines = this.DEFAULTS.skipEmptyLines;
        }
        if (typeof config.fastMode !== 'boolean') {
            config.fastMode = this.DEFAULTS.fastMode;
        }
        return config;
    }
    CsvToJson(_INPUT, _CONFIG) {
        const config = this.copyAndValidateConfig(_CONFIG);
        const ph = new sclCsvParserHandler.ParserHandle(config);
        const results = ph.parse(_INPUT);
        if (this.isFunction(config.complete)) {
            config.complete(results);
        }
        return results;
    }
}
/** @nocollapse */ /** @nocollapse */ SclUtilImporterService.ɵfac = function SclUtilImporterService_Factory(t) { return new (t || SclUtilImporterService)(i0.ɵɵinject(i1.FormbirdInjectorService)); };
/** @nocollapse */ /** @nocollapse */ SclUtilImporterService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclUtilImporterService, factory: SclUtilImporterService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclUtilImporterService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return [{ type: i1.FormbirdInjectorService }]; }, null);
})();

class SclAddressUtilService {
    constructor() { }
    resetAddressData(streetData) {
        streetData.properties.unitNo = '';
        streetData.properties.streetNo = '';
        streetData.properties.street = '';
        streetData.properties.state = '';
        streetData.properties.suburb = '';
        streetData.properties.postcode = '';
        streetData.properties.country = '';
        streetData.properties.locationDescription = '';
        streetData.geometry.coordinates[0] = '';
        streetData.geometry.coordinates[1] = '';
    }
    setAddressData(address, coordinates, resultObj) {
        if (address.unitNo) {
            resultObj.properties.unitNo = address.unitNo;
        }
        resultObj.properties.streetNo = address.street_number;
        resultObj.properties.street = address.street_name;
        resultObj.properties.suburb = address.locality;
        resultObj.properties.postcode = address.postal_code;
        resultObj.properties.country = address.country;
        resultObj.properties.state = address.admin_1;
        // var coordinates = tpl.coordinates;
        const lat = parseFloat(coordinates.latitude);
        const lon = parseFloat(coordinates.longitude);
        resultObj.geometry.coordinates[0] = lon;
        resultObj.geometry.coordinates[1] = lat;
        return { obj: resultObj, lat: lat, lon: lon };
    }
}
/** @nocollapse */ /** @nocollapse */ SclAddressUtilService.ɵfac = function SclAddressUtilService_Factory(t) { return new (t || SclAddressUtilService)(); };
/** @nocollapse */ /** @nocollapse */ SclAddressUtilService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclAddressUtilService, factory: SclAddressUtilService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclAddressUtilService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return []; }, null);
})();

function ValidationMessageComponent_div_0_span_1_Template(rf, ctx) {
    if (rf & 1) {
        i0.ɵɵelementStart(0, "span", 4);
        i0.ɵɵtext(1);
        i0.ɵɵelementEnd();
    }
    if (rf & 2) {
        const validator_r1 = i0.ɵɵnextContext().$implicit;
        i0.ɵɵadvance(1);
        i0.ɵɵtextInterpolate1(" ", validator_r1.text, " ");
    }
}
function ValidationMessageComponent_div_0_mat_icon_2_Template(rf, ctx) {
    if (rf & 1) {
        i0.ɵɵelementStart(0, "mat-icon", 5);
        i0.ɵɵtext(1, "error");
        i0.ɵɵelementEnd();
    }
    if (rf & 2) {
        const validator_r1 = i0.ɵɵnextContext().$implicit;
        i0.ɵɵattribute("aria-label", validator_r1.text);
    }
}
function ValidationMessageComponent_div_0_Template(rf, ctx) {
    if (rf & 1) {
        i0.ɵɵelementStart(0, "div", 1);
        i0.ɵɵtemplate(1, ValidationMessageComponent_div_0_span_1_Template, 2, 1, "span", 2);
        i0.ɵɵtemplate(2, ValidationMessageComponent_div_0_mat_icon_2_Template, 2, 1, "mat-icon", 3);
        i0.ɵɵelementEnd();
    }
    if (rf & 2) {
        const validator_r1 = ctx.$implicit;
        const ctx_r0 = i0.ɵɵnextContext();
        i0.ɵɵadvance(1);
        i0.ɵɵproperty("ngIf", ctx_r0.validatorValue[ctx_r0.documentId] && ctx_r0.validatorValue[ctx_r0.documentId][ctx_r0.fieldName] && ctx_r0.validatorValue[ctx_r0.documentId][ctx_r0.fieldName][validator_r1.name]);
        i0.ɵɵadvance(1);
        i0.ɵɵproperty("ngIf", ctx_r0.validatorValue[ctx_r0.documentId] && ctx_r0.validatorValue[ctx_r0.documentId][ctx_r0.fieldName] && ctx_r0.validatorValue[ctx_r0.documentId][ctx_r0.fieldName][validator_r1.name]);
    }
}
class ValidationMessageComponent {
    constructor(validationService, broadcastService, cdr) {
        this.validationService = validationService;
        this.broadcastService = broadcastService;
        this.cdr = cdr;
        this.subscriptions = new Subscription();
    }
    loadData() {
        const self = this;
        if (!self.model || !self.componentDefinition) {
            console.warn('No model or template item provided for component.');
            return;
        }
        self.fieldName = self.componentDefinition.name;
        self.documentId = self.model.documentId;
        self.validators = self.validationService.loadValidators(self.componentDefinition, self.fieldName);
        self.validatorValue = self.validationService.validationCtrlValues;
        const subscription = self.broadcastService.on(ClientConstants.VALIDATE_FORM_FIELDS)
            .subscribe((value) => {
            if (value && value.documentId === self.documentId && value.fieldName === self.fieldName) {
                this.cdr.detectChanges();
            }
        });
        self.subscriptions.add(subscription);
    }
    ngOnInit() {
        this.loadData();
    }
    ngOnDestroy() {
        this.subscriptions.unsubscribe();
    }
}
/** @nocollapse */ /** @nocollapse */ ValidationMessageComponent.ɵfac = function ValidationMessageComponent_Factory(t) { return new (t || ValidationMessageComponent)(i0.ɵɵdirectiveInject(i1.ValidationService), i0.ɵɵdirectiveInject(i1.BroadcastService), i0.ɵɵdirectiveInject(i0.ChangeDetectorRef)); };
/** @nocollapse */ /** @nocollapse */ ValidationMessageComponent.ɵcmp = /** @pureOrBreakMyCode */ i0.ɵɵdefineComponent({ type: ValidationMessageComponent, selectors: [["ft-validation-display"]], inputs: { model: "model", componentDefinition: "componentDefinition" }, decls: 1, vars: 1, consts: [["class", "validator", 4, "ngFor", "ngForOf"], [1, "validator"], ["class", "mat-error", "role", "alert", 4, "ngIf"], ["class", "ft-validation-icon-error mat-error", "attr.aria-hidden", "false", 4, "ngIf"], ["role", "alert", 1, "mat-error"], ["attr.aria-hidden", "false", 1, "ft-validation-icon-error", "mat-error"]], template: function ValidationMessageComponent_Template(rf, ctx) {
        if (rf & 1) {
            i0.ɵɵtemplate(0, ValidationMessageComponent_div_0_Template, 3, 2, "div", 0);
        }
        if (rf & 2) {
            i0.ɵɵproperty("ngForOf", ctx.validators);
        }
    }, directives: [i2.NgForOf, i2.NgIf, i3.MatIcon], styles: [".validator[_ngcontent-%COMP%]{position:relative;display:block}.validator[_ngcontent-%COMP%] > .mat-error[_ngcontent-%COMP%]:not(.ft-validation-icon-error){font-size:75%;padding-top:10px}.validator[_ngcontent-%COMP%] > .mat-error[_ngcontent-%COMP%]:not(.ft-validation-icon-error) + .ft-validation-icon-error[_ngcontent-%COMP%]{position:absolute;right:0;bottom:30px}@media screen and (max-width: 520.98px){.validator[_ngcontent-%COMP%]{margin-top:-15px;margin-bottom:15px}.validator[_ngcontent-%COMP%] > .mat-error[_ngcontent-%COMP%]:not(.ft-validation-icon-error){font-size:75%;padding-top:0}}"], changeDetection: 0 });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(ValidationMessageComponent, [{
            type: Component,
            args: [{ selector: 'ft-validation-display', changeDetection: ChangeDetectionStrategy.OnPush, template: "<div *ngFor=\"let validator of validators\" class=\"validator\">\n\n    <span class=\"mat-error\" role=\"alert\" *ngIf=\"validatorValue[documentId] &&\n\n                validatorValue[documentId][fieldName] &&\n                validatorValue[documentId][fieldName][validator.name]\">\n                {{ validator.text }}\n    </span>\n\n    <mat-icon class=\"ft-validation-icon-error mat-error\"\n    *ngIf=\"validatorValue[documentId] &&\n    validatorValue[documentId][fieldName] &&\n    validatorValue[documentId][fieldName][validator.name]\"\n    attr.aria-hidden=\"false\" [attr.aria-label]=\"validator.text\">error</mat-icon>\n</div>\n", styles: [".validator{position:relative;display:block}.validator>.mat-error:not(.ft-validation-icon-error){font-size:75%;padding-top:10px}.validator>.mat-error:not(.ft-validation-icon-error)+.ft-validation-icon-error{position:absolute;right:0;bottom:30px}@media screen and (max-width: 520.98px){.validator{margin-top:-15px;margin-bottom:15px}.validator>.mat-error:not(.ft-validation-icon-error){font-size:75%;padding-top:0}}\n"] }]
        }], function () { return [{ type: i1.ValidationService }, { type: i1.BroadcastService }, { type: i0.ChangeDetectorRef }]; }, { model: [{
                type: Input
            }], componentDefinition: [{
                type: Input
            }] });
})();

class ComponentSetupService {
    constructor() {
        this.formGroups = {};
    }
    setupStandardProperties(component) {
        const compDef = component.componentDefinition;
        if (compDef) {
            component.disabled = compDef.enabled === false;
        }
        component.elementName = component.fieldName;
        component.registeredControlName = component.fieldName + component.key;
    }
    /**
     * setup standard component variables. This can be called from the ngOnInit function of a component with:
     * this.componentService.setupStandardComponent(this);
     * @return the responsible control that holds the component value
     */
    setupStandardComponent(component, validators) {
        this.setupStandardProperties(component);
        const documentId = component.document.documentId;
        const formGroup = this.ensureFormGroupExisted(documentId);
        const compDef = component.componentDefinition;
        this.setupDefaultValidator(compDef, validators);
        const control = new FormControl({ value: component.fieldValue, disabled: this.isComponentDisabled(compDef) }, validators);
        return formGroup.registerControl(compDef.name + component.key, control);
    }
    setupControl(component, fieldName, fieldValue, validators) {
        const formGroup = this.ensureFormGroupExisted(component.document.documentId);
        const control = new FormControl(fieldValue, validators);
        return formGroup.registerControl(fieldName + component.key, control);
    }
    enableField(tplItem, formControl) {
        if (formControl) {
            if (tplItem.enabled === false) {
                formControl.disable({ emitEvent: false });
            }
            else {
                formControl.enable({ emitEvent: false });
            }
        }
    }
    enableFieldByFlag(enabled, formControl) {
        if (formControl) {
            if (enabled === false) {
                formControl.disable({ emitEvent: false });
            }
            else {
                formControl.enable({ emitEvent: false });
            }
        }
    }
    setupDefaultValidator(componentDefinition, validatorOpts) {
        validatorOpts = validatorOpts ? validatorOpts : [];
        if (componentDefinition.mandatory) {
            validatorOpts.push(Validators.required);
        }
    }
    isComponentDisabled(compDef) {
        return compDef.enabled === false;
    }
    ensureFormGroupExisted(documentId) {
        if (!this.formGroups[documentId]) {
            this.formGroups[documentId] = new FormGroup({});
        }
        return this.formGroups[documentId];
    }
    getControl(documentId, path) {
        if (this.formGroups[documentId]) {
            const formGroup = this.formGroups[documentId];
            return formGroup.get(path);
        }
        return null;
    }
}
/** @nocollapse */ /** @nocollapse */ ComponentSetupService.ɵfac = function ComponentSetupService_Factory(t) { return new (t || ComponentSetupService)(); };
/** @nocollapse */ /** @nocollapse */ ComponentSetupService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: ComponentSetupService, factory: ComponentSetupService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(ComponentSetupService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root',
                }]
        }], function () { return []; }, null);
})();

class FormbirdScSharedModule {
}
/** @nocollapse */ /** @nocollapse */ FormbirdScSharedModule.ɵfac = function FormbirdScSharedModule_Factory(t) { return new (t || FormbirdScSharedModule)(); };
/** @nocollapse */ /** @nocollapse */ FormbirdScSharedModule.ɵmod = /** @pureOrBreakMyCode */ i0.ɵɵdefineNgModule({ type: FormbirdScSharedModule });
/** @nocollapse */ /** @nocollapse */ FormbirdScSharedModule.ɵinj = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjector({ providers: [ComponentSetupService], imports: [[CommonModule, MatIconModule, FormbirdServicesModule, MatAutocompleteModule]] });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(FormbirdScSharedModule, [{
            type: NgModule,
            args: [{
                    declarations: [ValidationMessageComponent],
                    imports: [CommonModule, MatIconModule, FormbirdServicesModule, MatAutocompleteModule],
                    exports: [ValidationMessageComponent],
                    providers: [ComponentSetupService],
                }]
        }], null, null);
})();
(function () { (typeof ngJitMode === "undefined" || ngJitMode) && i0.ɵɵsetNgModuleScope(FormbirdScSharedModule, { declarations: [ValidationMessageComponent], imports: [CommonModule, MatIconModule, FormbirdServicesModule, MatAutocompleteModule], exports: [ValidationMessageComponent] }); })();

class SclUtilGmapService {
    constructor() { }
    initMapLabel(google) {
        function MapLabel(optOptions) {
            this.set('fontFamily', 'sans-serif');
            this.set('fontSize', 12);
            this.set('fontColor', '#000000');
            this.set('strokeWeight', 4);
            this.set('strokeColor', '#ffffff');
            this.set('align', 'center');
            this.set('zIndex', 1e3);
            this.setValues(optOptions);
        }
        MapLabel.prototype = new google.maps.OverlayView();
        // tslint:disable-next-line: no-string-literal
        window['MapLabel'] = MapLabel;
        /** @inheritDoc */
        MapLabel.prototype.changed = function (prop) {
            switch (prop) {
                case 'fontFamily':
                case 'fontSize':
                case 'fontColor':
                case 'strokeWeight':
                case 'strokeColor':
                case 'align':
                case 'text':
                    return this.drawCanvas_();
                case 'maxZoom':
                case 'minZoom':
                case 'position':
                    return this.draw();
            }
        };
        /**
         * Draws the label to the canvas 2d context.
         * @private
         */
        MapLabel.prototype.drawCanvas_ = function () {
            const canvas = this.canvas_;
            if (!canvas) {
                return;
            }
            const style = canvas.style;
            style.zIndex = /** @type number */ (this.get('zIndex'));
            const ctx = canvas.getContext('2d');
            ctx.clearRect(0, 0, canvas.width, canvas.height);
            ctx.strokeStyle = this.get('strokeColor');
            ctx.fillStyle = this.get('fontColor');
            ctx.font = this.get('fontSize') + 'px ' + this.get('fontFamily');
            const strokeWeight = Number(this.get('strokeWeight'));
            const text = this.get('text');
            if (text) {
                if (strokeWeight) {
                    ctx.lineWidth = strokeWeight;
                    ctx.strokeText(text, strokeWeight, strokeWeight);
                }
                ctx.fillText(text, strokeWeight, strokeWeight);
                const textMeasure = ctx.measureText(text);
                const textWidth = textMeasure.width + strokeWeight;
                style.marginLeft = this.getMarginLeft_(textWidth) + 'px';
                // Bring actual text top in line with desired latitude.
                // Cheaper than calculating height of text.
                style.marginTop = '-1em';
            }
        };
        /**
         * @inheritDoc
         */
        MapLabel.prototype.onAdd = function () {
            const canvas = this.canvas_ = document.createElement('canvas');
            const style = canvas.style;
            style.position = 'absolute';
            const ctx = canvas.getContext('2d');
            ctx.lineJoin = 'round';
            ctx.textBaseline = 'top';
            this.drawCanvas_();
            const panes = this.getPanes();
            if (panes) {
                panes.mapPane.appendChild(canvas);
            }
        };
        // tslint:disable-next-line: no-string-literal
        MapLabel.prototype['onAdd'] = MapLabel.prototype.onAdd;
        /**
         * Gets the appropriate margin-left for the canvas.
         * @private
         * @param {number} textWidth  the width of the text, in pixels.
         * @return {number} the margin-left, in pixels.
         */
        MapLabel.prototype.getMarginLeft_ = function (textWidth) {
            switch (this.get('align')) {
                case 'left':
                    return 0;
                case 'right':
                    return -textWidth;
            }
            return textWidth / -2;
        };
        /**
         * @inheritDoc
         */
        MapLabel.prototype.draw = function () {
            const projection = this.getProjection();
            if (!projection) {
                // The map projection is not ready yet so do nothing
                return;
            }
            if (!this.canvas_) {
                // onAdd has not been called yet.
                return;
            }
            const latLng = /** @type {google.maps.LatLng} */ (this.get('position'));
            if (!latLng) {
                return;
            }
            const pos = projection.fromLatLngToDivPixel(latLng);
            const style = this.canvas_.style;
            // tslint:disable-next-line: no-string-literal
            style['top'] = pos.y + 'px';
            // tslint:disable-next-line: no-string-literal
            style['left'] = pos.x + 'px';
            // tslint:disable-next-line: no-string-literal
            style['visibility'] = this.getVisible_();
        };
        // tslint:disable-next-line: no-string-literal
        MapLabel.prototype['draw'] = MapLabel.prototype.draw;
        /**
         * Get the visibility of the label.
         * @private
         * @return {string} blank string if visible, 'hidden' if invisible.
         */
        MapLabel.prototype.getVisible_ = function () {
            const minZoom = /** @type number */ (this.get('minZoom'));
            const maxZoom = /** @type number */ (this.get('maxZoom'));
            if (minZoom === undefined && maxZoom === undefined) {
                return '';
            }
            const map = this.getMap();
            if (!map) {
                return '';
            }
            const mapZoom = map.getZoom();
            if (mapZoom < minZoom || mapZoom > maxZoom) {
                return 'hidden';
            }
            return '';
        };
        /**
         * @inheritDoc
         */
        MapLabel.prototype.onRemove = function () {
            const canvas = this.canvas_;
            if (canvas && canvas.parentNode) {
                canvas.parentNode.removeChild(canvas);
            }
        };
        // tslint:disable-next-line: no-string-literal
        MapLabel.prototype['onRemove'] = MapLabel.prototype.onRemove;
        return MapLabel;
    }
    convertGPSToAddress(lat, lon, key, $q, $http) {
        const deferrer = $q.defer();
        const link = 'https://maps.googleapis.com/maps/api/geocode/json?latlng=' + lat + ',' + lon + '&key=' + key;
        $http.get(link).then(function (data) {
            if (data.data.status === 'OK') {
                deferrer.resolve(data);
            }
            else {
                deferrer.reject('Error: ' + data.data.error_message);
            }
        }).catch(function (error) {
            console.log('Error : ' + error);
            deferrer.reject(error);
        });
        return deferrer.promise;
    }
    getGeoCoors(SearchResultsProcessor, results, geozonesData, geozonesName, google, cb) {
        const finalData = [];
        SearchResultsProcessor.processSearchResults(results, (itemData) => {
            if (!itemData) {
                return;
            }
            // elementProcessingCallback
            const gCoors = [];
            const imeiGeoData = itemData[geozonesData];
            for (let j = 0; j < imeiGeoData.features.length; j++) {
                const geometry = imeiGeoData.features[j].geometry;
                const coordinates = geometry.coordinates;
                if (geometry.type === 'LineString') {
                    for (const key in coordinates) {
                        if (coordinates.hasOwnProperty(key)) {
                            const gCoor = new google.maps.LatLng(coordinates[key][1], coordinates[key][0]);
                            gCoors.push(gCoor);
                        }
                    }
                    finalData.push({ name: itemData[geozonesName],
                        coors: gCoors,
                        shape: 'Line' });
                }
                else if (geometry.type === 'Polygon') {
                    for (const key in coordinates[0]) {
                        if (coordinates[0].hasOwnProperty(key)) {
                            const gCoor = new google.maps.LatLng(coordinates[0][key][1], coordinates[0][key][0]);
                            gCoors.push(gCoor);
                        }
                    }
                    finalData.push({ name: itemData[geozonesName],
                        coors: gCoors,
                        shape: 'Polygon' });
                }
                else if (geometry.type === 'Point') {
                    const gCoor = new google.maps.LatLng(coordinates[1], coordinates[0]);
                    gCoors.push(gCoor);
                    finalData.push({ name: itemData[geozonesName],
                        coors: gCoors,
                        shape: 'Point' });
                }
                else if (geometry.type === 'MultiPolygon') {
                    for (let h = 0; h < coordinates.length; h++) {
                        for (const key in coordinates[h]) {
                            if (coordinates[h].hasOwnProperty(key)) {
                                const gCoor = new google.maps.LatLng(coordinates[h][key][1], coordinates[h][key][0]);
                                gCoors.push(gCoor);
                            }
                        }
                        finalData.push({ name: itemData[geozonesName],
                            coors: gCoors,
                            shape: 'Polygon' });
                    }
                }
                else {
                    console.log(geometry.type + ' is not handle yet');
                }
            }
        }, () => {
            // completionCallback
            console.log('the "processSearchResults" is complete');
        });
        if (cb && typeof cb === 'function') {
            cb(finalData);
        }
    }
}
/** @nocollapse */ /** @nocollapse */ SclUtilGmapService.ɵfac = function SclUtilGmapService_Factory(t) { return new (t || SclUtilGmapService)(); };
/** @nocollapse */ /** @nocollapse */ SclUtilGmapService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclUtilGmapService, factory: SclUtilGmapService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclUtilGmapService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return []; }, null);
})();

class SclUtilAvlService {
    constructor() { }
    getRelatedDocument(data, kind) {
        let retVal = {};
        if (data && data.imeiDocumentsRel && data.imeiDocumentsRel.length > 0 && kind) {
            retVal = data.imeiDocumentsRel.find(function (x) {
                if (!x || !x.imeiDocumentType) {
                    return false;
                }
                if (Array.isArray(x.imeiDocumentType)) {
                    const temp = x.imeiDocumentType.find(y => (y === kind));
                    if (temp) {
                        // logger.info("found event that have imeiDocumentType as array:");
                        // logger.info(data);
                        return true;
                    }
                    else {
                        return false;
                    }
                }
                else {
                    return x.imeiDocumentType === kind;
                }
            });
            // retVal = data.imeiDocumentsRel[0];
        }
        return retVal;
    }
}
/** @nocollapse */ /** @nocollapse */ SclUtilAvlService.ɵfac = function SclUtilAvlService_Factory(t) { return new (t || SclUtilAvlService)(); };
/** @nocollapse */ /** @nocollapse */ SclUtilAvlService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclUtilAvlService, factory: SclUtilAvlService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclUtilAvlService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return []; }, null);
})();

class CompressorBase30Service {
    constructor() {
        this.chunkSeparator = '_';
        this.charmap = {}; // {'1':'g','2':'h','3':'i','4':'j','5':'k','6':'l','7':'m','8':'n','9':'o','a':'p','b':'q','c':'r','d':'s','e':'t','f':'u','0':'v'}
        this.charmapReverse = {}; // will be filled by 'uncompress*" function
        this.minus = 'Z';
        this.plus = 'Y';
        this.acceptedformat = 'image/jsignature;base30';
        this.allchars = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWX'.split('');
        this.bitness = this.allchars.length / 2;
        for (let i = this.bitness - 1; i > -1; i--) {
            this.charmap[this.allchars[i]] = this.allchars[i + this.bitness];
            this.charmapReverse[this.allchars[i + this.bitness]] = this.allchars[i];
        }
    }
    remapTailChars(number) {
        // for any given number as string, returning string with trailing chars remapped something like so:
        // '345' -> '3de'
        const chars = number.split('');
        const l = chars.length;
        // we are skipping first char. standard hex number char = delimiter
        for (let i = 1; i < l; i++) {
            chars[i] = this.charmap[chars[i]];
        }
        return chars.join('');
    }
    compressstrokeleg(data) {
        // we convert half-stroke (only 'x' series or only 'y' series of numbers)
        // data is like this:
        // [517,516,514,513,513,513,514,516,519,524,529,537,541,543,544,544,539,536]
        // that is converted into this:
        // "5agm12100p1235584210m53"
        // each number in the chain is converted such:
        // - find diff from previous number
        // - first significant digit is kept as digit char. digit char = start of new number.
        // - consecutive numbers are mapped to letters, where 1 to 9 are A to I, 0 is O
        // Sign changes are denoted by "P" - plus, "M" for minus.
        const answer = [];
        let lastwhole = 0;
        const last = 0;
        let lastpolarity = 1;
        const l = data.length;
        let nwhole;
        let n;
        let absn;
        for (let i = 0; i < l; i++) {
            // we start with whole coordinates for each point
            // coords are converted into series of vectors:
            // [512, 514, 520]
            // [512, +2, +6]
            nwhole = Math.round(data[i]);
            n = nwhole - lastwhole;
            lastwhole = nwhole;
            // inserting sign change when needed.
            if (n < 0 && lastpolarity > 0) {
                lastpolarity = -1;
                answer.push(this.minus);
            }
            else if (n > 0 && lastpolarity < 0) {
                lastpolarity = 1;
                answer.push(this.plus);
            }
            // since we have dealt with sign. let's absolute the value.
            absn = Math.abs(n);
            // adding number to list  We convert these to Hex before storing on the string.
            if (absn >= this.bitness) {
                answer.push(this.remapTailChars(absn.toString(this.bitness)));
            }
            else {
                answer.push(absn.toString(this.bitness));
            }
        }
        return answer.join('');
    }
    uncompressstrokeleg(datastring) {
        // we convert half-stroke (only 'x' series or only 'y' series of numbers)
        // datastring like this:
        // "5agm12100p1235584210m53"
        // is converted into this:
        // [517,516,514,513,513,513,514,516,519,524,529,537,541,543,544,544,539,536]
        // each number in the chain is converted such:
        // - digit char = start of new whole number. Alpha chars except "p","m" are numbers in hiding.
        //   These consecutive digist expressed as alphas mapped back to digit char.
        //   resurrected number is the diff between this point and prior coord.
        // - running polaritiy is attached to the number.
        // - we undiff (signed number + prior coord) the number.
        // - if char 'm','p', flip running polarity
        const answer = [];
        const chars = datastring.split('');
        const l = chars.length;
        let ch;
        let polarity = 1;
        let partial = [];
        let preprewhole = 0;
        let prewhole;
        for (let i = 0; i < l; i++) {
            ch = chars[i];
            if (ch in this.charmap || ch === this.minus || ch === this.plus) {
                // this is new number - start of a new whole number.
                // before we can deal with it, we need to flush out what we already
                // parsed out from string, but keep in limbo, waiting for this sign
                // that prior number is done.
                // we deal with 3 numbers here:
                // 1. start of this number - a diff from previous number to
                //    whole, new number, which we cannot do anything with cause
                //    we don't know its ending yet.
                // 2. number that we now realize have just finished parsing = prewhole
                // 3. number we keep around that came before prewhole = preprewhole
                if (partial.length !== 0) {
                    // yep, we have some number parts in there.
                    prewhole = parseInt(partial.join(''), this.bitness) * polarity + preprewhole;
                    answer.push(prewhole);
                    preprewhole = prewhole;
                }
                if (ch === this.minus) {
                    polarity = -1;
                    partial = [];
                }
                else if (ch === this.plus) {
                    polarity = 1;
                    partial = [];
                }
                else {
                    // now, let's start collecting parts for the new number:
                    partial = [ch];
                }
            }
            else /* alphas replacing digits */ {
                // more parts for the new number
                partial.push(this.charmapReverse[ch]);
            }
        }
        // we always will have something stuck in partial
        // because we don't have closing delimiter
        answer.push(parseInt(partial.join(''), this.bitness) * polarity + preprewhole);
        return answer;
    }
    compressstrokes(data) {
        const answer = [];
        const l = data.length;
        let stroke;
        for (let i = 0; i < l; i++) {
            stroke = data[i];
            answer.push(this.compressstrokeleg(stroke.x));
            answer.push(this.compressstrokeleg(stroke.y));
        }
        return answer.join(this.chunkSeparator);
    }
    uncompressstrokes(datastring) {
        const data = [];
        const chunks = datastring.split(this.chunkSeparator);
        const l = chunks.length / 2;
        for (let i = 0; i < l; i++) {
            data.push({
                x: this.uncompressstrokeleg(chunks[i * 2]),
                y: this.uncompressstrokeleg(chunks[i * 2 + 1])
            });
        }
        return data;
    }
    pluginCompressor(data) {
        return [this.acceptedformat, this.compressstrokes(data)];
    }
    pluginDecompressor(data, formattype, importcallable) {
        if (typeof data !== 'string') {
            return;
        }
        if (data.substring(0, this.acceptedformat.length).toLowerCase() === this.acceptedformat) {
            data = data.substring(this.acceptedformat.length + 1); // chopping off "," there
        }
        importcallable(this.uncompressstrokes(data));
    }
}
/** @nocollapse */ /** @nocollapse */ CompressorBase30Service.ɵfac = function CompressorBase30Service_Factory(t) { return new (t || CompressorBase30Service)(); };
/** @nocollapse */ /** @nocollapse */ CompressorBase30Service.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: CompressorBase30Service, factory: CompressorBase30Service.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(CompressorBase30Service, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return []; }, null);
})();

class SclUtilAuditService {
    constructor() {
    }
    // Split string and then return to object array.(used in sc-audit-table)
    getObjectSplit(object, type) {
        let returnObject = object;
        if (type === 'doubleEquals') {
            returnObject = returnObject.split(':==:');
        }
        else if (type === 'equalWithArrow') {
            returnObject = returnObject.split('=:>');
        }
        return returnObject;
    }
    // Get list of object that has changes/ has an update .(used in sc-audit-table)
    getUpdatedObject(obj) {
        const objectChangesSring = '';
        const objectStrHoldter = [];
        for (const key in obj) {
            if (!(key === '_id' || key === 'systemHeader')) {
                if (this.getDataType(obj[key]) === 'array' || this.getDataType(obj[key]) === 'object') {
                    const objInside1 = obj[key];
                    for (const insideKey1 in objInside1) {
                        if (this.getDataType(objInside1[insideKey1]) === 'array' || this.getDataType(objInside1[insideKey1]) === 'object') {
                            const objInside2 = objInside1[insideKey1];
                            for (const insideKey2 in objInside2) {
                                if (this.getDataType(objInside2[insideKey2]) === 'array' || this.getDataType(objInside2[insideKey2]) === 'object') {
                                    const objInside3 = objInside2[insideKey2];
                                    for (const insideKey3 in objInside3) {
                                        if (this.getDataType(objInside3[insideKey3]) === 'array' || this.getDataType(objInside3[insideKey3]) === 'object') {
                                            // console.log("2: =>", key, insideKey1, insideKey2, insideKey3, +" -> " + JSON.stringify(objInside3[insideKey3]));
                                        }
                                        else {
                                            if (objInside3[insideKey3] === 'unchanged') {
                                                break;
                                            }
                                            objectStrHoldter.push(key + '=>' + insideKey1
                                                + '=>' + insideKey2 + ':==:' + insideKey3 + '=:>' + objInside3[insideKey3]);
                                        }
                                    }
                                }
                                else {
                                    if (objInside2[insideKey2] === 'unchanged') {
                                        break;
                                    }
                                    objectStrHoldter.push(key + '=>' + insideKey1 + ':==:' + insideKey2 + '=:>' + objInside2[insideKey2]);
                                }
                            }
                        }
                        else {
                            if (objInside1[insideKey1] === 'unchanged') {
                                break;
                            }
                            objectStrHoldter.push(key + ':==:' + insideKey1 + '=:>' + objInside1[insideKey1]);
                        }
                    }
                }
                else {
                    objectStrHoldter.push(key + '=:>' + obj[key]);
                }
            }
        }
        return objectStrHoldter;
    }
    // Split that return data type.(used in sc-audit-table)
    getDataType(data) {
        if (data === undefined || data === null) {
            return 'undefined';
        }
        if (data.constructor === String) {
            return 'string';
        }
        if (data.constructor === Array) {
            return 'array';
        }
        if (data.constructor === Object) {
            return 'object';
        }
        if (data.constructor === Number) {
            return 'number';
        }
        if (data.constructor === Boolean) {
            return 'boolean';
        }
        if (data.constructor === Function) {
            return 'function';
        }
        if (data.constructor === Date) {
            return 'date';
        }
        if (data.constructor === RegExp) {
            return 'regex';
        }
        return 'unknown';
    }
    // funtion return the difference of two object versions.(used in sc-audit-table)
    getObjectDifference(obj1, obj2) {
        if ((this.getDataType(obj1) !== 'array' && this.getDataType(obj1) !== 'object') ||
            (this.getDataType(obj2) !== 'array' && this.getDataType(obj2) !== 'object')) {
            let type1 = '';
            if (obj1 === obj2 || (this.getDataType(obj1) === 'date' && this.getDataType(obj2) === 'date' && obj1.getTime() === obj2.getTime())) {
                type1 = 'unchanged';
            }
            else if (this.getDataType(obj1) === 'undefined') {
                type1 = 'created';
            }
            if (this.getDataType(obj2) === 'undefined') {
                type1 = 'deleted';
            }
            else if (type1 === '') {
                type1 = 'updated';
            }
            return {
                type: type1,
                data: (obj1 === undefined) ? obj2 : obj1
            };
        }
        let diff;
        if (this.getDataType(obj1) === 'array' && this.getDataType(obj2) === 'array') {
            diff = [];
            obj1.sort();
            obj2.sort();
            for (let i1 = 0; i1 < obj2.length; i1++) {
                const type2 = obj1.indexOf(obj2[i1]) === -1 ? 'created' : 'unchanged';
                if (type2 === 'created' && (this.getDataType(obj2[i1]) === 'array' || this.getDataType(obj2[i1]) === 'object')) {
                    diff.push(this.getObjectDifference(obj1[i1], obj2[i1]));
                    continue;
                }
                diff.push({
                    type: type2,
                    data: obj2[i1]
                });
            }
            for (let i2 = 0; i2 < obj1.length; i2++) {
                if (obj2.indexOf(obj1[i2]) !== -1 || this.getDataType(obj1[i2]) === 'array' || this.getDataType(obj1[i2]) === 'object') {
                    continue;
                }
                diff.push({
                    type: 'deleted',
                    data: obj1[i2]
                });
            }
        }
        else {
            diff = {};
            let key = Object.keys(obj1);
            for (let i3 = 0; i3 < key.length; i3++) {
                let value2;
                if (this.getDataType(obj2[key[i3]]) !== 'undefined') {
                    value2 = obj2[key[i3]];
                }
                diff[key[i3]] = this.getObjectDifference(obj1[key[i3]], value2);
            }
            key = Object.keys(obj2);
            for (let i = 0; i < key.length; i++) {
                if (this.getDataType(diff[key[i]]) !== 'undefined') {
                    continue;
                }
                diff[key[i]] = this.getObjectDifference(undefined, obj2[key[i]]);
            }
        }
        return diff;
    }
    // function to get list of changes in all versions.(used in sc-audit-table)
    getObjectVersionData(objectDocVersionData, getUserFn, formatTimeFn, getLabelsFromTemplateFn, resultUserDataObj, currentTemplateComponentsObj, isHasTargetTableComponent) {
        const dataResult = objectDocVersionData;
        const objectDataSet = [];
        let dataChangesObj = [];
        const objectRecordChanges = [];
        const dataResultLength = dataResult.length - 1;
        for (let index = 0; index < dataResult.length; index++) {
            const insideDataSetObject = [];
            const stringType = '';
            const stringKey = '';
            const inc = 0;
            let varWho = '';
            let varWhen = '';
            let varInitialValue = '';
            let varNewValue = '';
            const objectVersion1 = dataResult[index];
            const nextIndex = index + 1;
            const objectVersion2 = dataResult[nextIndex];
            if (index === dataResultLength) {
                const createByEmailName1 = getUserFn(objectVersion1.systemHeader.createdBy, resultUserDataObj);
                varWho = createByEmailName1 && createByEmailName1.systemHeader ?
                    createByEmailName1.systemHeader.summaryName : objectVersion1.systemHeader.createdBy;
                varWhen = isHasTargetTableComponent ? objectVersion1.systemHeader.createdDate : formatTimeFn(objectVersion1.systemHeader.createdDate);
                varInitialValue = 'Created';
                varNewValue = 'Created';
                insideDataSetObject.push(index + 1);
                insideDataSetObject.push(varWho);
                insideDataSetObject.push(varWhen);
                insideDataSetObject.push(varInitialValue);
                insideDataSetObject.push(varNewValue);
                insideDataSetObject.push(objectVersion1);
                // insideDataSetObject.push(objectVersion1.systemHeader.systemType);
            }
            else {
                // compare two versionId incremented by 1 versionIndex
                dataChangesObj = this.getObjectDifference(objectVersion2, objectVersion1);
                let objectUpdated = this.getUpdatedObject(dataChangesObj);
                // console.log(index, objectVersion1, objectVersion2, dataChangesObj,objectUpdated);
                // console.log(objectUpdated.length,objectUpdated,dataChangesObj);
                objectRecordChanges.push(objectUpdated);
                const newObjectUpdated = [];
                for (const keyObjUpdated1 in objectUpdated) {
                    // Filter objectUpdated list, return string type and not string data
                    if (!(objectUpdated[keyObjUpdated1].toString().indexOf('data=') >= 0)) {
                        newObjectUpdated.push(objectUpdated[keyObjUpdated1]);
                    }
                }
                objectUpdated = newObjectUpdated;
                // console.log(objectRecordChanges,objectUpdated.length,objectUpdated,dataChangesObj);
                let strNewValue = '';
                let strOldValue = '';
                let thisTypeValue;
                for (const keyObjUpdated in objectUpdated) {
                    if (objectUpdated.length > 0) {
                        const keyNames = objectUpdated[keyObjUpdated].split(':==:')[0].split('=>');
                        // console.log(keyNames);
                        strOldValue = (strOldValue.trim() === '' ? '' : strOldValue + '\n');
                        strNewValue = (strNewValue.trim() === '' ? '' : strNewValue + '\n');
                        if (keyNames.length === 1) {
                            thisTypeValue = dataChangesObj[keyNames[0]].type;
                            if (thisTypeValue === 'created') {
                                strOldValue = strOldValue + getLabelsFromTemplateFn(keyNames[0], currentTemplateComponentsObj) + ':No initial value';
                                strNewValue = strNewValue + getLabelsFromTemplateFn(keyNames[0], currentTemplateComponentsObj) + ':' + JSON.stringify(dataChangesObj[keyNames[0]].data)
                                    + ' (New)';
                            }
                            else if (thisTypeValue === 'deleted') {
                                strOldValue = strOldValue + getLabelsFromTemplateFn(keyNames[0], currentTemplateComponentsObj) + ':' + JSON.stringify(objectVersion2[keyNames[0]]);
                                strNewValue = strNewValue + getLabelsFromTemplateFn(keyNames[0], currentTemplateComponentsObj) + ' (Deleted)';
                            }
                            else if (thisTypeValue === 'updated') {
                                strOldValue = strOldValue + getLabelsFromTemplateFn(keyNames[0], currentTemplateComponentsObj) + ':' + JSON.stringify(objectVersion2[keyNames[0]]);
                                strNewValue = strNewValue + getLabelsFromTemplateFn(keyNames[0], currentTemplateComponentsObj) + ':' + JSON.stringify(objectVersion1[keyNames[0]]) + ' (Updated)';
                            }
                        }
                        else if (keyNames.length === 2) {
                            thisTypeValue = dataChangesObj[keyNames[0]][keyNames[1]].type;
                            if (thisTypeValue === 'created') {
                                strOldValue = strOldValue + getLabelsFromTemplateFn(keyNames[0], currentTemplateComponentsObj) + '.' + keyNames[1] + ':No initial value';
                                strNewValue = strNewValue + getLabelsFromTemplateFn(keyNames[0], currentTemplateComponentsObj) + '.' + keyNames[1] + ':'
                                    + JSON.stringify(objectVersion1[keyNames[0]][keyNames[1]]) + ' (New)';
                            }
                            else if (thisTypeValue === 'deleted') {
                                strOldValue = strOldValue + getLabelsFromTemplateFn(keyNames[0], currentTemplateComponentsObj) + '.' + keyNames[1] + ':'
                                    + JSON.stringify(objectVersion2[keyNames[0]][keyNames[1]]);
                                strNewValue = strNewValue + getLabelsFromTemplateFn(keyNames[0], currentTemplateComponentsObj) + '.' + keyNames[1] + ' (Deleted)';
                            }
                            else if (thisTypeValue === 'updated') {
                                strOldValue = strOldValue + getLabelsFromTemplateFn(keyNames[0], currentTemplateComponentsObj) + '.' + keyNames[1] + ':'
                                    + JSON.stringify(objectVersion2[keyNames[0]][keyNames[1]]);
                                strNewValue = strNewValue + getLabelsFromTemplateFn(keyNames[0], currentTemplateComponentsObj) + '.' + keyNames[1]
                                    + ':' + JSON.stringify(objectVersion1[keyNames[0]][keyNames[1]]) + ' (Updated)';
                            }
                        }
                        else if (keyNames.length === 3) {
                            thisTypeValue = dataChangesObj[keyNames[0]][keyNames[1]][keyNames[2]].type;
                            if (thisTypeValue === 'created') {
                                strOldValue = strOldValue + keyNames[0] + '.' + keyNames[1] + '.' + keyNames[2] + ':No initial value';
                                strNewValue = strNewValue + keyNames[0] + '.' + keyNames[1] + '.' + keyNames[2] + ':'
                                    + JSON.stringify(objectVersion1[keyNames[0]][keyNames[1]][keyNames[2]]) + ' (New)';
                            }
                            else if (thisTypeValue === 'deleted') {
                                strOldValue = strOldValue + keyNames[0] + '.' + keyNames[1] + '.' + keyNames[2] + ':'
                                    + JSON.stringify(objectVersion2[keyNames[0]][keyNames[1]][keyNames[2]]);
                                strNewValue = strNewValue + keyNames[0] + '.' + keyNames[1] + '.' + keyNames[2] + ' (Deleted)';
                            }
                            else if (thisTypeValue === 'updated') {
                                strOldValue = strOldValue + keyNames[0] + '.' + keyNames[1] + '.' + keyNames[2] + ':'
                                    + JSON.stringify(objectVersion2[keyNames[0]][keyNames[1]][keyNames[2]]);
                                strNewValue = strNewValue + keyNames[0] + '.' + keyNames[1] + '.' + keyNames[2] + ':'
                                    + JSON.stringify(objectVersion1[keyNames[0]][keyNames[1]][keyNames[2]]) + ' (Updated)';
                            }
                        }
                    }
                }
                const createByEmailName1 = getUserFn(objectVersion1.systemHeader.createdBy, resultUserDataObj);
                varWho = createByEmailName1 && createByEmailName1.systemHeader ?
                    createByEmailName1.systemHeader.summaryName : objectVersion1.systemHeader.createdBy;
                varWhen = isHasTargetTableComponent ? objectVersion1.systemHeader.createdDate :
                    formatTimeFn(objectVersion1.systemHeader.createdDate);
                varInitialValue = strOldValue;
                varNewValue = strNewValue;
                insideDataSetObject.push(index + 1);
                insideDataSetObject.push(varWho);
                insideDataSetObject.push(varWhen);
                insideDataSetObject.push(varInitialValue + '\n' + 'lastUpdated:"' + objectVersion2.systemHeader.createdDate + '"');
                insideDataSetObject.push(varNewValue + '\n' + 'lastUpdated:"' + objectVersion1.systemHeader.createdDate + '"');
                insideDataSetObject.push(objectVersion1);
                // insideDataSetObject.push(objectVersion1.systemHeader.systemType);
            }
            objectDataSet.push(insideDataSetObject);
        }
        return objectDataSet;
    }
}
/** @nocollapse */ /** @nocollapse */ SclUtilAuditService.ɵfac = function SclUtilAuditService_Factory(t) { return new (t || SclUtilAuditService)(); };
/** @nocollapse */ /** @nocollapse */ SclUtilAuditService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclUtilAuditService, factory: SclUtilAuditService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclUtilAuditService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return []; }, null);
})();

// This is clone of util-sharing.service but have broadcast service
class UtilSharingNgService {
    constructor(clientRulesService, broadcastService) {
        this.clientRulesService = clientRulesService;
        this.broadcastService = broadcastService;
    }
    // Function that would trigger/call functions from one component to other components/ rules
    //  componentFieldName=>name of component that contains the function;functionName=> name of the function declared in componentFieldName
    // e.g component function call:scope.sclUtilSharing.fnCallFunction(scope.fieldName,'eventReloadDatatable');
    // e.g rules calling: UtilSharingLibrary.fnCallFunction('scDatatablesTest','eventReloadDatatable');
    fnCallFunction(componentFieldName, functionName, param1, param2) {
        const value = { fieldName: componentFieldName, param1, param2, time: new Date() };
        this.broadcastService.broadcast(functionName, value);
        this.clientRulesService.setSharedValue(functionName, value);
        if (/get/.test(functionName.slice(0, 3))) {
            const objectData = this.clientRulesService.getSharedValue(functionName.replace(/^get/, 'set'));
            let objectDataReturn = [];
            if (objectData) {
                if (objectData.param1 && objectData.param2) {
                    objectDataReturn.push(objectData.param1, objectData.param2);
                }
                else if (objectData.param1) {
                    objectDataReturn = objectData.param1;
                }
                else {
                    objectDataReturn = objectData;
                }
                return objectDataReturn;
            }
        }
    }
    // Function that would register a list of function to call in fnCallFunction
    // listOfFunctions=>['eventReloadDatatable','eventReloadTable','function1','...']
    // list of functions that will be available  in fnCallFunction to be called.
    // listOfFunctions = [{functionName, function}, ...];
    fnRegisterFunctions(listOfFunctions, fieldName, documentId) {
        for (let i = 0, n = listOfFunctions.length; i < n; i++) {
            this.fnCreateWatchForFunctionCall(listOfFunctions[i].name, listOfFunctions[i].fn, fieldName, documentId);
        }
    }
    fnCreateWatchForFunctionCall(functionName, fn, fieldName, documentId) {
        this.broadcastService.on(functionName).subscribe((objValue) => {
            // console.log('%c broadcast on, functionName = ' + functionName, 'color:blue');
            if (objValue) {
                // logger.info(functionName," is watch",objValue,objValue.fieldName,scope.fieldName);
                if (fieldName === objValue.fieldName) {
                    // logger.info(objValue.fieldName,scope.fieldName,variableFunction,fn);
                    if (objValue.param1 && objValue.param2) {
                        // variableFunction = "scope."+functionName+"('"+objValue.param1+"','"+objValue.param2+"')";
                        fn(objValue.param1, objValue.param2);
                    }
                    else if (objValue.param1) {
                        // variableFunction = "scope."+functionName+"('"+objValue.param1+"')";
                        fn(objValue.param1);
                    }
                    else {
                        fn();
                    }
                    // eval("(" + variableFunction + ")");
                }
            }
        });
    }
    fnPassedDataObject(fieldName, objectName, objectDataValue) {
        this.clientRulesService.setSharedValue(objectName, { fieldName, objectDataValue, time: new Date().getTime() });
    }
    fnGetDataObject(fieldName, objectName) {
        const objectData = this.clientRulesService.getSharedValue(objectName);
        return objectData.fieldName === fieldName ? objectData : null;
    }
}
/** @nocollapse */ /** @nocollapse */ UtilSharingNgService.ɵfac = function UtilSharingNgService_Factory(t) { return new (t || UtilSharingNgService)(i0.ɵɵinject(i1.ClientRulesService), i0.ɵɵinject(i1.BroadcastService)); };
/** @nocollapse */ /** @nocollapse */ UtilSharingNgService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: UtilSharingNgService, factory: UtilSharingNgService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(UtilSharingNgService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return [{ type: i1.ClientRulesService }, { type: i1.BroadcastService }]; }, null);
})();

class SclSearchClientConstantService {
}
SclSearchClientConstantService.operator = {
    equals: 'eq',
    notEquals: 'neq',
    contains: 'contains'
};

const logger = console;
class SclSearchClientService {
    constructor(httpClient, formbirdInjectorService, searchService, configService, dataService) {
        this.httpClient = httpClient;
        this.formbirdInjectorService = formbirdInjectorService;
        this.searchService = searchService;
        this.configService = configService;
        this.dataService = dataService;
        this.processDexieResults = (options, results, resolve) => __awaiter(this, void 0, void 0, function* () {
            let docs = [];
            let filterDocs = results.data;
            if (filterDocs && filterDocs.length > 0) {
                filterDocs = yield this.filterResults(options, filterDocs);
                if (filterDocs && filterDocs.length > 0) {
                    filterDocs = this.sortResults(options, filterDocs);
                    const size = options.size || options.pageSize || SharedConstants.DEFAULT_PAGE_SIZE;
                    let pageFrom;
                    if (!options.page && options.from && options.from > 0 && options.size > 0) {
                        pageFrom = options.from / options.size;
                    }
                    const page = options.page || pageFrom || SharedConstants.DEFAULT_PAGE;
                    const docsArray = chunk(filterDocs, size);
                    docs = nth(docsArray, page);
                }
            }
            results.total = filterDocs.length;
            results.data = docs;
            resolve(results);
        });
        this.executeComponentFunction = (options, resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            const params = {
                functionName: options.functionName ? options.functionName : 'searchByQueryName',
                componentWebServiceName: options.componentWebServiceName ? options.componentWebServiceName : 'ws-search-service',
                functionParameters: [options.queryName, options.dataSource, options.document, options],
                offlineComponentLibraryName: options.offlineComponentLibraryName ? options.offlineComponentLibraryName : options.queryName
            };
            let docs = [];
            this.dataService.executeComponentFunction(params)
                .then((results) => {
                let data;
                if (results.data && results.data.hits && results.data.hits.hits) {
                    data = results.data.hits.hits;
                }
                else {
                    data = results;
                    results.total = data.length;
                    results.searchProvier = options.dataSource;
                    results.statusCode = 200;
                }
                if (data && data.length > 0) {
                    const size = options.size || options.pageSize || SharedConstants.DEFAULT_PAGE_SIZE;
                    let pageFrom;
                    if (!options.page && options.from && options.from > 0 && options.size > 0) {
                        pageFrom = options.from / options.size;
                    }
                    const page = options.page || pageFrom || SharedConstants.DEFAULT_PAGE;
                    const docsArray = chunk(data, size);
                    docs = nth(docsArray, page);
                }
                if (results.data && results.data.hits && results.data.hits.hits) {
                    results.data.hits.hits = docs;
                }
                else {
                    results.data = docs;
                }
                resolve(results);
            }, (error) => {
                reject(error);
            });
        });
        this.loggedInUserService = this.formbirdInjectorService.get('LoggedInUserService');
    }
    search(options) {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            try {
                if (!isEmpty(options.queryName) && options.dataSource) {
                    this.executeComponentFunction(options, resolve, reject);
                }
                else {
                    if (!isEmpty(options.queryName)) {
                        options.searchOptions = options.searchOptions ? options.searchOptions : {};
                        options.searchOptions.dexieSearchFunction = yield this.getDexieSearchFunction(options);
                    }
                    this.searchService.search(options).then((results) => {
                        if (results.searchProvier === 'dexie') {
                            this.processDexieResults(options, results, resolve);
                        }
                        else {
                            resolve(results);
                        }
                    }, (err) => {
                        reject(err);
                    });
                }
            }
            catch (error) {
                reject(error);
            }
        }));
    }
    getDexieSearchFunction(options) {
        const self = this;
        return new Promise((resolve, reject) => {
            this.getDexieQuery(options.queryName, function (err, queryDocument) {
                if (err) {
                    reject(err);
                }
                else if (queryDocument && !isEmpty(queryDocument.filter)) {
                    let fun;
                    try {
                        const funcStr = sclHandlebars.processHandlebars(queryDocument.filter, null, self.contextParams);
                        // tslint:disable-next-line:no-eval
                        fun = eval('(' + funcStr + ')');
                        resolve(fun);
                    }
                    catch (err) {
                        reject(err);
                    }
                }
                else {
                    resolve();
                }
            });
        });
    }
    getDexieQuery(queryName, callback) {
        const queryDocument = (db, callback) => __awaiter(this, void 0, void 0, function* () {
            try {
                yield db.documents.where('name').equals(queryName).and((document) => {
                    return document.systemHeader.systemType === 'query'
                        && toLower(document.searchProvider) === 'dexie'
                        && document.systemHeader.currentVersion === true;
                }).toArray((documents) => {
                    callback(null, documents);
                });
            }
            catch (err) {
                logger.error(err);
                callback(err);
            }
        });
        const options = { "searchOptions": { "dexieSearchFunction": queryDocument } };
        this.searchService.search(options).then((results) => {
            let qryDoc;
            if (results && results.data && results.data.length > 0) {
                qryDoc = results.data[0];
            }
            callback(null, qryDoc);
        }, (err) => {
            logger.error(err);
            callback(err);
        });
    }
    filterResults(options, documents) {
        return __awaiter(this, void 0, void 0, function* () {
            const self = this;
            return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                let query = [];
                let filterObj = options.filter;
                if (typeof filterObj === 'string') {
                    filterObj = filterObj.replace(/'/g, '"');
                    filterObj = JSON.parse(filterObj);
                }
                if (filterObj && !isEmpty(filterObj.dexieQuery) && isArray(filterObj.dexieQuery) && filterObj.dexieQuery.length > 0) {
                    query = cloneDeep(filterObj.dexieQuery);
                }
                const currentVerFilter = { field: 'systemHeader.currentVersion', value: true,
                    operator: SclSearchClientConstantService.operator.equals };
                query.push(currentVerFilter);
                const results = yield documents.filter(function (document) {
                    let isFound = false;
                    for (let i = 0; i < query.length; i++) {
                        const queryObj = query[i];
                        if (queryObj && queryObj.field && queryObj.value) {
                            const fldValue = get(document, queryObj.field);
                            if (isEmpty(queryObj.operator) || queryObj.operator === SclSearchClientConstantService.operator.contains) {
                                isFound = includes(toLower(fldValue), queryObj.value);
                            }
                            else if (queryObj.operator === SclSearchClientConstantService.operator.equals) {
                                isFound = isEqual(fldValue, queryObj.value) || isEqual(toLower(fldValue), queryObj.value);
                            }
                            else if (queryObj.operator === SclSearchClientConstantService.operator.notEquals) {
                                isFound = !isEqual(toLower(fldValue), queryObj.value);
                            }
                        }
                        if (!isFound) {
                            break;
                        }
                    }
                    return isFound;
                });
                resolve(results);
            }));
        });
    }
    sortResults(options, documents) {
        let filterObj = options.filter;
        if (typeof filterObj === 'string') {
            filterObj = filterObj.replace(/'/g, '"');
            filterObj = JSON.parse(filterObj);
        }
        if (filterObj && filterObj.dexieSort && isPlainObject(filterObj.dexieSort)) {
            let fieldArr;
            fieldArr = [];
            let valueArr;
            valueArr = [];
            map(filterObj.dexieSort, function (value, key) {
                fieldArr.push(key);
                valueArr.push(value);
            });
            documents = orderBy(documents, fieldArr, valueArr);
        }
        return documents;
    }
    getDefaultSearchProvider(context) {
        const offlineMode = this.loggedInUserService.getUserConfigItem('cachingEnabled') === true;
        const clientConfig = this.configService.clientConfig();
        this.contextParams = {
            document: context.document,
            doc: context.document,
            account: context.account,
            tpl: context.template,
            template: context.template,
            tplItem: context.componentDefinition,
            componentDefinition: context.componentDefinition // the current components configuration details
        };
        if (!isEmpty(context.componentDefinition.queryName) && offlineMode && context.componentDefinition.searchOfflineOnly) {
            return 'dexie';
        }
        else if (context.componentDefinition.searchProvider) {
            return context.componentDefinition.searchProvider;
        }
        else if (context.template.searchProvider) {
            return context.componentDefinition.searchProvider;
        }
        else if (clientConfig.defaultSearchProvider) {
            return clientConfig.defaultSearchProvider;
        }
        else {
            return null;
        }
    }
    isDexieQuery(context, queryName) {
        const offlineMode = this.loggedInUserService.getUserConfigItem('cachingEnabled') === true;
        return ((!isEmpty(context.componentDefinition.queryName) || queryName) &&
            (offlineMode || context.componentDefinition.searchOfflineOnly ||
                (context.componentDefinition.searchOptions && context.componentDefinition.searchOptions.searchOfflineOnly)));
    }
}
/** @nocollapse */ /** @nocollapse */ SclSearchClientService.ɵfac = function SclSearchClientService_Factory(t) { return new (t || SclSearchClientService)(i0.ɵɵinject(i1$1.HttpClient), i0.ɵɵinject(i1.FormbirdInjectorService), i0.ɵɵinject(i1.SearchService), i0.ɵɵinject(i1.ConfigService), i0.ɵɵinject(i1.DataService)); };
/** @nocollapse */ /** @nocollapse */ SclSearchClientService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclSearchClientService, factory: SclSearchClientService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclSearchClientService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return [{ type: i1$1.HttpClient }, { type: i1.FormbirdInjectorService }, { type: i1.SearchService }, { type: i1.ConfigService }, { type: i1.DataService }]; }, null);
})();

class SclWrappingService {
    constructor() { }
    processWrappingOpen(intervalObj, wrapActionDefinition, parent, elemOpenId, key, title, bgColor, fgColor, borderColor, enableMatFormFix, wrapHeading, wrapClass, iconClass, iconStyle, fullwidth, parentNode, wrappingCloseFn, updateUIFn) {
        let wrapAction = '';
        if (!(typeof wrapActionDefinition === 'undefined' || wrapActionDefinition == null)) {
            wrapAction = wrapActionDefinition;
        }
        if (wrapAction === 'open') {
            if (intervalObj == null) {
                if (parent != null) {
                    clearInterval(intervalObj);
                    intervalObj = null;
                    const parentId = parent.getAttribute('id');
                    if (parentId) {
                        parent.setAttribute('id', elemOpenId);
                        parent.classList.add(parentId);
                    }
                    parent.setAttribute('key', key);
                    parent.setAttribute('static-html-wrap', elemOpenId);
                    parent.setAttribute('static-html-wrap-heading-title', title ? title : '');
                    parent.setAttribute('static-html-wrap-heading-bg', bgColor);
                    parent.setAttribute('static-html-wrap-heading-fg', fgColor);
                    parent.setAttribute('static-html-wrap-border', borderColor);
                    if (enableMatFormFix) {
                        parent.classList.add('enable-mat-form-fix');
                    }
                    let isWrapHeading = true;
                    if (!(wrapHeading == null)) {
                        isWrapHeading = wrapHeading;
                    }
                    parent.setAttribute('static-html-wrap-heading', isWrapHeading);
                    parent.setAttribute('static-html-wrap-class', wrapClass);
                    parent.setAttribute('static-html-icon-class', iconClass);
                    parent.setAttribute('static-html-icon-style', iconStyle);
                    parent.setAttribute('static-html-wrap-fullwidth', fullwidth);
                    if (updateUIFn && typeof updateUIFn === 'function') {
                        updateUIFn();
                    }
                    if (wrappingCloseFn && typeof wrappingCloseFn === 'function') {
                        wrappingCloseFn();
                    }
                }
                else {
                    parent = parentNode;
                }
            }
        }
        return { wrapAction, intervalObj };
    }
    handlePanelIcon(open, panel) {
        const wrapId = open.getAttribute('id');
        const iconClass = open.getAttribute('static-html-icon-class');
        const hasIconClass = iconClass && iconClass !== 'undefined';
        const iconStyle = open.getAttribute('static-html-icon-style');
        const hasIconStyle = iconStyle && iconStyle !== 'undefined';
        if (hasIconClass || hasIconStyle) {
            const icon = document.createElement('div');
            const iconDiv = document.createElement('div');
            iconDiv.setAttribute('id', `circled${wrapId}`);
            iconDiv.className = 'fbPanelBackgroundCircle';
            const iWrap = document.createElement('i');
            if (hasIconClass) {
                iWrap.classList.add('ico');
                iWrap.classList.add(...iconClass.split(' '));
                iWrap.classList.add('fa-fw', 'defaultIconClass');
                panel.classList.add('hasIcon');
            }
            if (hasIconStyle) {
                iWrap.style.cssText = iconStyle;
            }
            iconDiv.appendChild(iWrap);
            icon.appendChild(iconDiv);
            panel.appendChild(icon);
        }
    }
    handlePanelHead(open, panel, className) {
        const wrapId = open.getAttribute('id');
        const wrapHeading = open.getAttribute('static-html-wrap-heading');
        const headingHtml = open.getAttribute('static-html-wrap-heading-title');
        const bg = open.getAttribute('static-html-wrap-heading-bg');
        const fg = open.getAttribute('static-html-wrap-heading-fg');
        const border = open.getAttribute('static-html-wrap-border');
        const head = document.createElement('div');
        head.className = className; // 'sc-panel-head'
        head.setAttribute('id', wrapId + 'head');
        head.innerHTML = headingHtml;
        if (bg !== '') {
            head.style.background = bg;
        }
        if (fg !== '') {
            head.style.color = fg;
        }
        if (border !== '') {
            panel.style.borderColor = border;
            head.style.borderColor = border;
        }
        if (wrapHeading === 'true') {
            panel.appendChild(head);
        }
    }
    handlePanelBody(open, panel, className, isStaticHtmlComp) {
        const wrapId = open.getAttribute('id');
        const body = document.createElement('div');
        body.className = className; // 'sc-panel-body';
        if (isStaticHtmlComp) {
            body.style.display = 'flex';
            body.style['flex-wrap'] = 'wrap';
        }
        body.setAttribute('id', wrapId + 'collapsable');
        panel.appendChild(body);
    }
    handlePanel(parent, open, close, classNames, isStaticHtmlComp) {
        const wrapId = open.getAttribute('id');
        const panel = document.createElement('div');
        panel.setAttribute('id', `${wrapId}panel`);
        panel.setAttribute('close-class', close.className);
        panel.style.padding = "0px";
        const wrap_class = (open.getAttribute('static-html-wrap-class') !== 'undefined') ? open.getAttribute('static-html-wrap-class') : classNames.wrapClassDefault;
        let panelClass = 'panel ft-panel-wrap ' + wrap_class + ' ' + open.className.replace('ng-hide', '');
        if (isStaticHtmlComp) {
            panelClass = panelClass.replace(/ft-grid-col-[0-9]/g, 'ft-grid-col-12');
        }
        panel.className = panelClass;
        this.handlePanelIcon(open, panel);
        this.handlePanelHead(open, panel, classNames.head);
        this.handlePanelBody(open, panel, classNames.collapsable, isStaticHtmlComp);
        parent.firstChild.appendChild(panel);
    }
    processWrappingClose(isStaticHtmlComp, elemOpenId, parent, siblings, siblingsGrid, classNames) {
        let success = false;
        let headId = `#${elemOpenId}head`;
        try {
            const open = document.getElementById(elemOpenId);
            const close = parent;
            let wrapElement = open;
            while (wrapElement) {
                if (wrapElement.nextSibling === close) {
                    break;
                }
                wrapElement = wrapElement.nextSibling;
                if (wrapElement == null) {
                    continue;
                }
                siblings.push(wrapElement);
                if (wrapElement && wrapElement.className) {
                    if (wrapElement.className.match(/ft-grid-col-[0-9]/g)) {
                        siblingsGrid += parseInt(wrapElement.className.match(/ft-grid-col-[0-9]/g)[0].split('-')[3], 10);
                    }
                    else if (wrapElement.className === 'ft-component-wrap ng-untouched ng-pristine ng-valid ng-star-inserted') {
                        siblingsGrid += 3;
                    }
                }
                else {
                    // siblingsGrid += parseInt(wrapElement.className.match(/col-lg-[0-9]/g)[0].split("-")[3]);
                }
            }
            const openFullwidth = open.getAttribute('static-html-wrap-fullwidth').toLowerCase();
            let newFtGrid = openFullwidth === 'false' ? (siblingsGrid >= 12 ? 12 : siblingsGrid) : 12;
            if (!isStaticHtmlComp) {
                open.className = open.className.replace(/ft-grid-col-[0-9]/g, 'ft-grid-col-' + newFtGrid); // panel-collapse
            }
            open.className = open.className.replace(/col-lg-[0-9]/g, 'col-lg-' + 12);
            this.handlePanel(parent, open, close, classNames, isStaticHtmlComp);
            success = true;
        }
        catch (err) {
            console.log('Encounter error. Going back to start wrapp ing');
            siblings = [];
        }
        return { siblings, siblingsGrid, success, headId };
    }
}
/** @nocollapse */ /** @nocollapse */ SclWrappingService.ɵfac = function SclWrappingService_Factory(t) { return new (t || SclWrappingService)(); };
/** @nocollapse */ /** @nocollapse */ SclWrappingService.ɵprov = /** @pureOrBreakMyCode */ i0.ɵɵdefineInjectable({ token: SclWrappingService, factory: SclWrappingService.ɵfac, providedIn: 'root' });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SclWrappingService, [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], function () { return []; }, null);
})();

/** Error when invalid control is dirty, touched, or submitted. */
class MyErrorStateMatcher {
    isErrorState(control, form) {
        const isSubmitted = form && form.submitted;
        return !!(control && control.invalid && (control.dirty || control.touched || isSubmitted));
    }
}
class AbstractComponent {
    constructor(formbirdInjectorService, changedDocumentService, componentSetupService, cdr) {
        this.formbirdInjectorService = formbirdInjectorService;
        this.changedDocumentService = changedDocumentService;
        this.componentSetupService = componentSetupService;
        this.cdr = cdr;
        /**
         * A sub-class could avoid calling notifyComponentIntialized on super-class by disabling this flag,
         * they will call that function at the end of their ngOnInit in stead
         * @default true to enable
         */
        this.shouldNotifyCompInitializedInSuper = true;
        this.shouldDispatchEmptyFieldValue = true;
        this.subs = new Subscription();
        this.matcher = new MyErrorStateMatcher();
        this.broadcastService = formbirdInjectorService.get('BroadcastService');
    }
    ngOnInit() {
        this.setupComponent();
        this.initializeFieldValue();
        this.setupChangeDetection();
        if (this.shouldNotifyCompInitializedInSuper) {
            this.notifyComponentIntialized();
        }
    }
    ngOnDestroy() {
        this.subs.unsubscribe();
    }
    setupComponent() {
        this.formControl = this.componentSetupService.setupStandardComponent(this, this.validators);
        this.setFieldActivation(false);
    }
    setupChangeDetection(avoidDebounceTime) {
        if (this.fieldName) {
            const docSub = this.changedDocumentService.watch(this.fieldName, this.document.documentId, this.registeredControlName)
                .subscribe(fieldValue => {
                console.log('On field value changed for component: ', this.registeredControlName);
                this.onFieldValueChanged(fieldValue);
            });
            this.subs.add(docSub);
        }
        const compSub = this.changedDocumentService.watchComponentDef(this.fieldName, this.template.documentId).subscribe(compDef => {
            this.onComponentDefChanged(compDef);
        });
        this.subs.add(compSub);
        if (this.formControl) {
            let changeSub;
            if (avoidDebounceTime) {
                changeSub = this.formControl.valueChanges.subscribe(value => {
                    this.onControlValueChanged(value);
                });
            }
            else {
                changeSub = this.formControl.valueChanges.pipe(debounceTime(ClientConstants.VALIDATION_INTERVAL_TIME)).subscribe(value => {
                    this.onControlValueChanged(value);
                });
            }
            this.subs.add(changeSub);
        }
        this.setupDocumentCreatedDetection();
    }
    setFieldActivation(shouldDetectChanges) {
        this.componentSetupService.enableField(this.componentDefinition, this.formControl);
        this.detectChanges(shouldDetectChanges);
    }
    /**
     * Call this function to dispatch the field value change action to store
     */
    valueChanged(value) {
        this.fieldValue = value;
        this.changedDocumentService.valueChanged(this, value, this.registeredControlName);
    }
    /**
     * Call this function to set the field value into store. Param isInitValue is for initializing value and does not trigger undo/redo
     */
    dispatchChanged(isInitValue) {
        const documentId = this.document.documentId;
        const documentChangedInfo = {
            documentId: documentId,
            fieldName: this.fieldName,
            newValue: this.fieldValue,
            isInitValue: isInitValue
        };
        this.changedDocumentService.notifyDocumentChanged(documentChangedInfo);
    }
    detectChanges(shouldDetectChange) {
        if (shouldDetectChange !== false && !this.cdr.destroyed) {
            this.cdr.detectChanges();
        }
    }
    updateComponentDefChange(componentDefinition) {
        this.componentDefinition = componentDefinition;
    }
    /**
     * This function will be called once user interacts with the control that manages the field value in the component.
     */
    onControlValueChanged(value) {
        this.valueChanged(value);
    }
    /**
     * This function will be called once the template component definition in store gets changed.
     */
    onComponentDefChanged(componentDefinition) {
        console.log("Change detection on template: ", componentDefinition.name);
        this.updateComponentDefChange(componentDefinition);
        const disabled = componentDefinition.enabled === false;
        if (this.disabled !== disabled) {
            this.disabled = disabled;
            this.setFieldActivation();
        }
    }
    /**
     * This function will be called once the field value in store gets changed.
     */
    onFieldValueChanged(fieldValue, shouldDetectChanges = true) {
        if (this.fieldValue === fieldValue) {
            return;
        }
        this.fieldValue = fieldValue;
        if (this.formControl) {
            this.formControl.patchValue(this.fieldValue, { emitEvent: false, onlySelf: true });
        }
        if (shouldDetectChanges) {
            this.detectChanges();
        }
    }
    setupDocumentCreatedDetection() {
        const docCreatedSub = this.broadcastService.on(ClientConstants.DOCUMENT_NEW).subscribe((documentInfo) => {
            this.onDocumentCreated(documentInfo);
            this.detectChanges();
        });
        this.subs.add(docCreatedSub);
    }
    /**
     * This function will be called to initialize field value such as: default value... once the New Document button's clicked to create a new document.
     * @param documentInfo includes documentId and unsavedDocumentListId
     */
    onDocumentCreated(documentInfo) {
        const document = documentInfo === null || documentInfo === void 0 ? void 0 : documentInfo.document;
        if (document) {
            const unsavedDocumentListId = documentInfo === null || documentInfo === void 0 ? void 0 : documentInfo.unsavedDocumentListId;
            const documentId = document.documentId;
            if (unsavedDocumentListId === this.formParameters.unsavedDocumentListId && this.document.documentId === documentId) {
                // set fieldValue as the origninal field value object
                this.fieldValue = document[this.fieldName];
                // setup fieldValue same as the component initialize in ngOnInit
                this.initializeFieldValue();
                // notify store the component's initialized
                this.notifyComponentIntialized();
            }
        }
    }
    /**
     * This function will initialize the field value including default value if needed
     */
    initializeFieldValue() {
    }
    /**
     * notify store the component has been initialized
     */
    notifyComponentIntialized() {
        this.changedDocumentService.notifyIntializedField(this, this.shouldDispatchEmptyFieldValue);
    }
    /**
     * Notify the component definition changes to store
     */
    notifyComponentDefChanged() {
    }
}
/** @nocollapse */ /** @nocollapse */ AbstractComponent.ɵfac = function AbstractComponent_Factory(t) { return new (t || AbstractComponent)(i0.ɵɵdirectiveInject(i1.FormbirdInjectorService), i0.ɵɵdirectiveInject(i1.ChangedDocumentService), i0.ɵɵdirectiveInject(ComponentSetupService), i0.ɵɵdirectiveInject(i0.ChangeDetectorRef)); };
/** @nocollapse */ /** @nocollapse */ AbstractComponent.ɵcmp = /** @pureOrBreakMyCode */ i0.ɵɵdefineComponent({ type: AbstractComponent, selectors: [["ng-component"]], inputs: { document: "document", fieldValue: "fieldValue", template: "template", fieldName: "fieldName", formParameters: "formParameters", key: "key", responsiveLayouts: "responsiveLayouts", message: "message", componentDefinition: "componentDefinition" }, decls: 0, vars: 0, template: function AbstractComponent_Template(rf, ctx) { }, encapsulation: 2 });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(AbstractComponent, [{
            type: Component,
            args: [{
                    template: '' // template will be overridden in child component
                }]
        }], function () { return [{ type: i1.FormbirdInjectorService }, { type: i1.ChangedDocumentService }, { type: ComponentSetupService }, { type: i0.ChangeDetectorRef }]; }, { document: [{
                type: Input
            }], fieldValue: [{
                type: Input
            }], template: [{
                type: Input
            }], fieldName: [{
                type: Input
            }], formParameters: [{
                type: Input
            }], key: [{
                type: Input
            }], responsiveLayouts: [{
                type: Input
            }], message: [{
                type: Input
            }], componentDefinition: [{
                type: Input
            }] });
})();

class AbstractDocumentComponent extends AbstractComponent {
    constructor(formbirdInjectorService, changedDocumentService, componentSetupService, cdr) {
        super(formbirdInjectorService, changedDocumentService, componentSetupService, cdr);
        this.formbirdInjectorService = formbirdInjectorService;
        this.changedDocumentService = changedDocumentService;
        this.componentSetupService = componentSetupService;
        this.cdr = cdr;
        this.subs = new Subscription();
    }
    ngOnInit() {
        super.ngOnInit();
        const docSub = this.changedDocumentService.watchDocument(this.document.documentId).subscribe(doc => {
            // console.log('On document change from store: ', this.document.documentId);
            this.document = doc;
        });
        this.subs.add(docSub);
    }
}
/** @nocollapse */ /** @nocollapse */ AbstractDocumentComponent.ɵfac = function AbstractDocumentComponent_Factory(t) { return new (t || AbstractDocumentComponent)(i0.ɵɵdirectiveInject(i1.FormbirdInjectorService), i0.ɵɵdirectiveInject(i1.ChangedDocumentService), i0.ɵɵdirectiveInject(ComponentSetupService), i0.ɵɵdirectiveInject(i0.ChangeDetectorRef)); };
/** @nocollapse */ /** @nocollapse */ AbstractDocumentComponent.ɵcmp = /** @pureOrBreakMyCode */ i0.ɵɵdefineComponent({ type: AbstractDocumentComponent, selectors: [["ng-component"]], features: [i0.ɵɵInheritDefinitionFeature], decls: 0, vars: 0, template: function AbstractDocumentComponent_Template(rf, ctx) { }, encapsulation: 2 });
(function () {
    (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(AbstractDocumentComponent, [{
            type: Component,
            args: [{
                    template: '' // template will be overridden in child component
                }]
        }], function () { return [{ type: i1.FormbirdInjectorService }, { type: i1.ChangedDocumentService }, { type: ComponentSetupService }, { type: i0.ChangeDetectorRef }]; }, null);
})();

/*
 * Public API Surface of formbird-sc-shared
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AbstractComponent, AbstractDocumentComponent, ComponentSetupService, CompressorBase30Service, CustomElementService, DeviceDetectService, FormbirdScSharedModule, MyErrorStateMatcher, SclAceEditorService, SclAceTextRulesInterfaceService, SclAddressUtilService, SclCsvParserHandlerService, SclCsvParserService, SclDatatableResizeService, SclDatatableResponsiveService, SclExportService, SclHandlebarsService, SclSearchClientService, SclUtilAuditService, SclUtilAvlService, SclUtilDateService, SclUtilDocumentService, SclUtilFilterService, SclUtilGmapService, SclUtilImporterService, SclUtilUrlService, SclWrappingService, UtilDocumentIdService, UtilSharingNgService, UtilSharingService, ValidationMessageComponent, sclHandlebars };
//# sourceMappingURL=formbird-sc-shared.mjs.map
